// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#version 330 core

layout(location = 0) in vec2 position;
layout(location = 1) in vec2 neighbour;
// Position of the vertex in the current streamline [0, streamlineLength-1]
layout(location = 2) in float localID;

uniform sampler2D texture0; // physical
uniform sampler2D texture1; // mask
uniform sampler2D texture2; // gcp up vector

uniform vec2 uvMin;
uniform vec2 uvRange;
uniform float speed;
uniform float headID;
uniform vec4 scaleFactor;
uniform vec4 offset;
uniform float glFactor;
uniform vec2 uvTexelStep;
uniform float rangeMax;
uniform bvec2 seamless;

// New position to pass to the fragment shader
out vec2 position0;

void main()
{
	// Check if it's the streamline head or a part of the body
    if (localID == headID)
    {
        // It's the head, compute her new position with the value of the dataset.
		vec2 pos = position;

		vec2 particlePos = (pos - uvMin) / uvRange;

        for (int i = 0; i < 20; ++i)
        {
		    // Read the mask
		    float mask = texture(texture1, position).x;

		    // Check if there are data here.
		    if (mask > 0.0)
		    {
			    // Data is available
			    // Read the vector in the dataset
			    vec2 dir = ((texture(texture0, particlePos).rg * glFactor) * scaleFactor.rg) + offset.rg;
		
			    float value = length(dir);
			    value = min(value, rangeMax) / rangeMax;

			    // North and East expressed in texels
                vec4 toto = texture(texture2, position);
			    vec2 vY = toto.xy;  // North
			    vec2 vX = toto.zw;  // East

			    dir = normalize(dir.x * vX + dir.y * vY) * value;

			    // Compute the new position
			    pos = pos + dir * uvTexelStep * speed;

			    if (seamless.x)
                {
                    if (pos.x >= 1.0)
                    {
                        pos.x -= 1.0;
                    }
                    else if (pos.x < 0.0)
                    {
                        pos.x += 1.0;
                    }
                }
                else if (seamless.y)
                {
                    if (pos.y >= 1.0)
                    {
                        pos.y -= 1.0;
                    }
                    else if (pos.y < 0.0)
                    {
                        pos.y += 1.0;
                    }
                }
                else
                {
                    if (pos.x >= 1.0 || pos.y >= 1.0)
                    {
                        pos = position;
                        break;
                    }

                    if (pos.x <= 0.0 || pos.y <= 0.0)
                    {
                        pos = position;
                        break;
                    }
                }

                pos.x = min(pos.x, 1.0);
                pos.y = min(pos.y, 1.0);

		    }
		    else
		    {
			    particlePos = pos;
                break;
		    }
        }

		// Save the result
		position0 = pos;
	}
	else
    {
        position0 = neighbour;
    }
}
