// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef BBOX_H
#define BBOX_H

class BBox
{
public:
    BBox();
    BBox(const glm::vec2& min, const glm::vec2& max);
    BBox(const glm::vec4& minmax);
    BBox(const glm::vec2 lonlat[4]);

    const glm::vec2& getMin() const {return _min;}
    const glm::vec2& getMax() const {return _max;}
    glm::vec4 getMinMax() const {return glm::vec4(_min, _max);}
    float getWidth() const {return _max.x - _min.x;}
    float getHeight() const {return _max.y - _min.y;}

    void setColor(const glm::vec4& color) {_color = color;}
    const glm::vec4& getColor() const {return _color;}

private:
    glm::vec2 _min;
    glm::vec2 _max;
    glm::vec4 _color;
};

#endif /* BBOX_H */
