// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   grid.h
 * Author: gleseach
 *
 * Created on 26 août 2015, 11:23
 */

#ifndef GRID_H
#define	GRID_H

void createGrid(const glm::vec3 points[4], int subdivisionX, int subdivisionY, std::vector<glm::vec3>& vertices, std::vector<glm::vec2>& uv, std::vector<unsigned short>& indices);


void createQuad(std::vector<glm::vec3>& vertices, std::vector<glm::vec2>& uv, float size);

void createGridOnGlobe(float * lons, unsigned int lon_size, float * lats, unsigned int lat_size, int precision, std::vector<glm::vec3>& vertices, std::vector<glm::vec2>& uv, std::vector<unsigned int>& indices);

void createGridOnPlan(float * lons, unsigned int lon_size, float * lats, unsigned int lat_size, int precision, std::vector<glm::vec3>& vertices, std::vector<glm::vec2>& uv, std::vector<unsigned int>& indices);

void createGridFromGCP(float * lons, unsigned int lon_size, float * lats, unsigned int lat_size, unsigned short * gcpx, unsigned short * gcpy, unsigned int width, unsigned int height, std::vector<glm::vec3>& vertices, std::vector<glm::vec2>& uv, std::vector<unsigned int>& indices);

void createGridOnPlanFromGCP(float * lons,
                       unsigned int lon_size,
                       float * lats,
                       unsigned int lat_size,
                       unsigned short * gcpx,
                       unsigned short * gcpy,
                       unsigned int width,
                       unsigned int height,
                       std::vector<glm::vec3>& vertices,
                       std::vector<glm::vec2>& uv,
                       std::vector<unsigned int>& indices);

/*!
 * \brief Convert a longitude/latitude vector in a xyz cartesian coordinates vector.
 */
void fromLonlatToSphereUnitVector(const std::vector<glm::vec2>& lonlats, std::vector<glm::vec3>& xyz);

#endif	/* GRID_H */

