// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef APPLICATION_H
#define APPLICATION_H

#include <cstdint>
#include <string>

#include "core/interfaces/isettings_store.h"
#include "core/interfaces/istate_store.h"
#include "core/interfaces/iwindow.h"
#include "core/threads_pool.h"
#include "core/data_scheduler.h"
#include "core/processing_scheduler.h"
#include "core/fifo_scheduler.h"
#include "3d/resources.h"
#include "core/engine.h"

class SceneView;

namespace SEAScope
{
    // Forward declarations
    class AppEventsDispatcher;

	class Application
	{
		public:
			Application(AppEventsDispatcher&);
			virtual ~Application();

			bool init(const std::string&, const std::string&, ISettingsStore&, IStateStore&, IWindow&, RendererResources&, const std::string&);
			bool initWorkspace(const std::string&);
			bool loadSettings(const std::string&, const std::string&);
			bool saveSettings();
			bool loadState();
			bool saveState();
			bool initRendererResources(const std::string&, RendererResources&) const;
			bool initWindow();
			bool initWorkers();
			bool openControlSocket(const std::string&, std::uint64_t);
			bool run(SceneView&);
			bool shutdown();

            AppEventsDispatcher& getEventsDispatcher();

            std::string settingsPath;
			ISettingsStore* settingsStore;
			IStateStore* stateStore;
			IWindow* window;

            Engine engine;

			DataScheduler dataScheduler;
			ThreadsPool dataWorkers;
			ProcessingScheduler processScheduler;
			ThreadsPool processWorkers;
			FIFOScheduler internalScheduler;
			ThreadsPool internalWorkers;
	};
};

#endif
