// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
/* 
 * File:   copy.h
 * Author: gleseach
 *
 * Created on 31 août 2015, 17:22
 */

#ifndef COPY_H
#define	COPY_H

#include "core/log.h"

/* must be included in the header because it contains templates */
#include "vendors/_glm.h"

void memcpy2D(void * src, unsigned src_width, unsigned src_height, unsigned int element_size, void * dst, unsigned int origin_x, unsigned int origin_y, unsigned dst_width, unsigned dst_height);

template<typename T>
void memcpy2D(T * src, const glm::ivec2& src_size, unsigned int element_count, T* dst, const glm::ivec2& dst_size, const glm::ivec2& origin)
{
    SEAScope::Log::log(SEAScope::LogLevel::Debug, "Copy", "memcpy2D element size {}", sizeof(T) * element_count);

    memcpy2D(
            src,
            src_size.x,
            src_size.y,
            sizeof(T) * element_count,
            dst,
            origin.x,
            origin.y,
            dst_size.x,
            dst_size.y
            );
}

#endif	/* COPY_H */

