// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef DATA_MEDIATOR_H
#define DATA_MEDIATOR_H

#include <string>
#include <set>
#include <functional>
#include "core/interfaces/idata_store.h"
#include "core/sources_control_model.h"
#include "core/threads_pool.h"

namespace SEAScope
{
	class DataMediator
	{
		public:
			DataMediator();
			virtual ~DataMediator();

			bool init(IDataStore*, SourcesControlModel*);
			bool loadData(const GranuleId&, const std::string&, const std::set<std::string>&, DataModel, const std::function<void()>&);
			void dataLoaded(const GranuleId&, const std::string&, const std::set<std::string>&);

			IDataStore* store;
			SourcesControlModel* sources;
			ThreadsPool* workers;
	};
};

#endif
