// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ICATALOGUE_SERVICE_CONNECTOR_H
#define ICATALOGUE_SERVICE_CONNECTOR_H

#include <cstdint>
#include <vector>
#include <set>

#include "core/types.h"
#include "core/structs.h"
#include "core/settings.h"
#include "core/rendering_cfg.h"

namespace SEAScope
{
	class ICatalogueServiceConnector
	{
		public:
			virtual ~ICatalogueServiceConnector();

			virtual bool init(const Settings&) = 0;
            virtual bool reset() = 0;
			virtual bool listCollections(std::set<std::uint32_t>&) const = 0;
			virtual bool getCollectionCfg(std::uint32_t, CollectionCfg&, std::vector<RenderingCfg*>&) const = 0;
            virtual bool canHandle(const std::string&) const = 0;
			virtual std::string getPath(const CollectionId& id) const = 0;

			/* Required by plugin system */
			static const char* api()
            {
                return "catalogue";
            }
	};
};

#endif
