// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef LOG_H
#define LOG_H

#include "vendors/_fmt.h"

#include <string>
#include "core/interfaces/ilog_handler.h"

namespace SEAScope
{
	class Log
	{
		public:


			static bool log(const LogLevel&, const std::string&, const std::string&);

			template<typename ...Details>
			static bool log(const LogLevel& level, const std::string& logger, const std::string& msgTemplate, const Details&... details)
			{
				if
					(nullptr == Log::implementation)
				{
					return false;
				}

                if
                    (false == Log::implementation->isMessageValid(level, logger))
                {
                    return false;
                }

				std::string msg;
				msg = fmt::format(msgTemplate, details...);
				return Log::log(level, logger, msg);
			}

            static LogLevel levelFromString(const std::string& logLevelStr)
            {
                if
                    ("debug" == logLevelStr)
                {
                    return LogLevel::Debug;
                }
                else if
                    ("info" == logLevelStr)
                {
                    return LogLevel::Info;
                }
                else if
                    ("warn" == logLevelStr)
                {
                    return LogLevel::Warn;
                }
                else if
                    ("error" == logLevelStr)
                {
                    return LogLevel::Error;
                }
                else if
                    ("fatal" == logLevelStr)
                {
                    return LogLevel::Fatal;
                }

                return LogLevel::Info;
            }

			static ILogHandler* implementation;
	};
};

#endif
