// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef TOOLS_H
#define TOOLS_H

#include "tools/window_controller.h"
#include "tools/scene_view_controller_original.h"
#include "tools/debug_controller.h"
#include "tools/extraction_controller.h"
#include "tools/transect_controller.h"
#include "tools/selection_controller.h"

namespace SEAScope
{
    // Forward declarations
    class AppEventsDispatcher;

    class Tools
    {
        public:
            Tools(AppEventsDispatcher&);
            virtual ~Tools();

            WindowController* getWindowController();
            SceneViewControllerBase* getSceneViewController();
            DebugController* getDebugController();
            ExtractionController* getExtractionController();
            TransectController* getTransectController();
            SelectionController* getSelectionController();

        protected:
            WindowController _windowController;
            SceneViewControllerOriginal _sceneViewController;
            DebugController _debugController;
            ExtractionController _extractionController;
            TransectController _transectController;
            SelectionController _selectionController;
    };
}
#endif
