// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef TYPES_H
#define TYPES_H

#include <cstdint>
#include <vector>
#include <chrono>
#include <string>
#include <array>
#include <tuple>
#include <map>
#include "data/image.h"

namespace SEAScope
{
	typedef std::pair<std::uint32_t, std::uint64_t> GranuleId;
	typedef std::pair<std::uint32_t, std::uint32_t> CollectionId;
	typedef std::vector<std::array<std::uint8_t, 3>> Colormap;
	typedef std::uint32_t VariableId;
	typedef std::uint32_t SourceId;

	typedef std::uint64_t timestamp_t;

	typedef std::pair<CollectionId, VariableId> CollectionVariable;

	typedef std::int32_t socket_t;

	typedef std::chrono::high_resolution_clock::time_point DisplayBatchId;

	struct DataBucket;
	typedef std::map<std::string, DataBucket> GranuleData;

    enum class RenderableType:std::uint8_t {
        POLYGON = 0,
        POLYLINE = 1,
        ELLIPSE = 2,
        POINT = 3,
        GRANULE = 4
    };

	enum class RenderMethod:std::uint8_t {
		RASTER = 0,
		ARROWS = 1,
		BARBS = 2,
		TRAJECTORIES = 3,
		DOTSCLOUD = 4,
		STREAMLINES = 5,
		RAWRGB = 6,
        POLYLINE = 7,
        POLYGON = 8,
        ELLIPSE = 9,
        POINT = 10
	};

	enum class RenderingParam:std::uint8_t {
		LOGSCALE = 0,
		MIN = 1,
		MAX = 2,
		OPACITY = 3,
		ZINDEX = 4,
		LABEL = 5,
		COLORMAP = 6,
		RENDER_METHOD = 7,
		PARTICLES_COUNT = 8,
		PARTICLE_TTL = 9,
		STREAMLINE_LENGTH = 10,
		STREAMLINE_SPEED = 11,
		RENDERED = 12,
		SCALE = 13,
		ANY = 14,
		COLOR = 15,
		FILTER_MODE = 16
	};

	enum class DisplayJobType : std::uint8_t {
        NOOP = 0,
		UPDATE_STARTED = 1,
		LOAD_DATA = 2,
		UPDATE_RENDERING = 3,
		UPDATE_COMPLETE = 4
	};

    enum class DataModel : std::uint8_t {
        LAT_LON = 0,
        Y_X = 1,
        NJ_NI = 2,
        ROW_CELL = 3,
        TIME_STATION = 4,
        TIME = 5
    };

    enum class IDFAttributes : std::uint16_t {
        TITLE = 0,
        INSTITUTION = 1,
        COMMENT = 2,
        FILE_ID = 3,
        PRODUCT_VERSION = 4,
        LAT_MIN = 5,
        LAT_MAX = 6,
        LON_MIN = 7,
        LON_MAX = 8,
        CREATOR_EMAIL = 9,
        STATION_ID = 10,
        PLATFORM = 11,
        SENSOR = 12,
        COUNT
    };

	enum class FilterMode : std::uint8_t {
		NEAREST = 0,
		BILINEAR = 1
	};

    enum class RenderableState : std::uint8_t {
        NORMAL = 0,
        SELECTED = 1,
        EDITION = 2
    };

	enum ShapeType
	{
		POLYLINE = 0,
		POLYGON = 1,
		CIRCLE = 2,
		RECTANGLE = 3,
        ELLIPSE = 4,
        ELLIPSE2 = 5,
        POINT = 6,
		UNKNOWN_SHAPE_TYPE = 0xFF
	};

	typedef void* (*PluginInstanciator)();
	typedef bool (*PluginDestroyer)(void*);

    typedef std::uint32_t ActorId;
    typedef std::uint32_t ActionType;
    enum class Synchronicity : std::uint8_t {
        NONE = 0,
        SYNC = 1,
        ASYNC = 2,
        COUNT
    };
};

#endif
