// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef WORKSPACE_FOLDERS_H
#define WORKSPACE_FOLDERS_H

#include "3d/singleton.h"

class WorkspaceFolders : public Singleton<WorkspaceFolders>
{
public:
	friend class Singleton<WorkspaceFolders>;

	bool setWorkspacePath(const std::string& path);

	// All paths have a separator character at the end.
	const std::string& getImageFolderPath() const;
	const std::string& getIconFolderPath() const;
	const std::string& getShaderFolderPath() const;
	const std::string& getbBarbsFolderPath() const;
	const std::string& getbArrowsFolderPath() const;
	const std::string& getStyleFilePath() const;

private:
    WorkspaceFolders();
    ~WorkspaceFolders() override;

	std::string _workspacePath;
	std::string _imagePath;
	std::string _arrowsPath;
	std::string _barbsPath;
	std::string _shaderPath;
	std::string _iconPath;
	std::string _styleFilePath;
};

#endif // WORKSPACE_FOLDERS_H
