// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef DATA_HANDLER_H
#define DATA_HANDLER_H

#include "vendors/_glm.h"

class DataSet;
class Image;

namespace SEAScope
{
    class DataHandler
    {
        public:
            DataHandler();
            virtual ~DataHandler();

            /*!
             * \brief Parse the picture and generate a mask according to the fillValue.
             * \param[in] data The data to parse.
             * \param[in] fillValue The value corresponding to an invalid data in the picture.
             * \param[out] mask The generated mask.
             */
            static void computeMask(const Image& data, float fillValue, Image& mask);

            /*!
             * \brief Apply a dilation on the picture according to the mask borders.
             */
            static void dilate(DataSet * data_set);

            static glm::vec4 computeBbox(float * lons, float * lats, unsigned short * gcpx, unsigned short * gcpy, unsigned int lon_size, unsigned int lat_size, bool& lonsGrowing, bool& latsGrowing, bool& sameVariationForLonAndX, bool& sameVariationForLatAndY);

			template<typename T>
			static void createMaskBorder(Image&, Image&, float*, float*);

            static void exportToBinary(const std::string&, DataSet*);
    };
};

#endif
