// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef TRANSECT_REQUEST_H
#define TRANSECT_REQUEST_H

#include <map>
#include <string>

#include "core/app_event.h"

// Forward declarations
class ShapeMesh;
class SceneView;

namespace SEAScope
{
    // Forward declarations
    struct RenderableId;
    struct Extraction;

    class TransectRequest: public AppEvent
    {
        public:
            TransectRequest(SceneView*, ShapeMesh*, std::map<RenderableId, Extraction>*, std::map<RenderableId, std::string>*);
            virtual ~TransectRequest() override;

            SceneView* getSceneView() const;
            ShapeMesh* getFootprint() const;
            std::map<RenderableId, Extraction>* getExtractedData() const;
            std::map<RenderableId, std::string>* getExtractedLabels() const;

        protected:
            SceneView* _sceneView;
            ShapeMesh* _footprint;
            std::map<RenderableId, Extraction>* _extractedData;
            std::map<RenderableId, std::string>* _extractedLabels;
    };
}

#endif
