// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef GRANULES_INDEX_H
#define GRANULES_INDEX_H

#include <map>
#include <set>
#include <array>
#include <mutex>
#include <string>
#include <cstdint>
#include <functional>

#include "core/structs.h"
#include "data/events_coverage.h"

namespace SEAScope
{
	class GranulesIndex
	{
		public:
			GranulesIndex();
			virtual ~GranulesIndex();

			bool hasGranule(const std::string&) const;
			bool hasGranule(std::uint64_t) const;
			bool add(GranuleMetadata&, bool);
            bool updateTimeCoverage(std::uint64_t, const timestamp_t, const timestamp_t);
            bool remove(const std::string&, bool);
			std::array<bool, 2> adjacentGranules( const timestamp_t
												, const timestamp_t
												, const std::set<std::uint32_t>&
												, GranuleMetadata&
												, GranuleMetadata&) const;
			bool closestGranuleBound( const timestamp_t
									, const std::set<std::uint32_t>&
									, GranuleMetadata&) const;
			bool getCoverage(const std::set<std::uint32_t>&, EventsCoverage&) const;
			bool getCoverage(std::uint32_t, EventsCoverage&) const;
			bool findGranules(const SearchCriteria&, std::vector<GranuleMetadata>&) const;
            std::uint64_t generateGranuleId();

            std::mutex mapsMutex;
			std::uint64_t idGenerator;
			std::map<std::uint64_t, GranuleMetadata> data;
			std::map<std::string, std::uint64_t> ids;
			std::function<void(std::uint64_t, std::uint32_t)> granuleAddedCallback;
	};
};

#endif
