// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef IN_MEMORY_DATA_STORE_H
#define IN_MEMORY_DATA_STORE_H

#include <map>
#include <mutex>
#include <cstdint>

#include "core/interfaces/idata_store.h"

namespace SEAScope
{
	class InMemoryDataStore final : public virtual IDataStore
	{
		public:
			InMemoryDataStore();
			~InMemoryDataStore() override;

			GranuleData& getDataBucket(const std::string&) override;
			Image* getData(const std::string&, const std::set<std::string>&, DataInfo&, bool&) override;
		    bool hasData(const std::string&, const std::string&) const override;

			bool getFieldData(const std::string&, const std::string&, DataInfo&, Image&) const;

            std::mutex granuleDataMutex;
			std::map<std::string, GranuleData> data;
			std::map<std::string, DataBucket> compositeData;
	};
};

#endif
