// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef ANNOTATIONS_CONTROLLER_H
#define ANNOTATIONS_CONTROLLER_H

#include <set>
#include <string>
#include <unordered_map>

#include "core/interfaces/iapp_events_subscriber.h"
#include "core/rendering_cfg.h"
#include "local/user_generated.h"

// Forward declarations
class SceneView;
class Icons;

namespace OGL
{
    // Forward declarations
    class Shape;
}

struct ShapeTypeHash
{
    template<typename T>
    std::size_t operator()(T t) const
    {
        return static_cast<std::size_t>(t);
    }
};

namespace SEAScope
{
    // Forward declarations
    class AppEventsDispatcher;
    class UserGenerated;
    class IAnnotationsStore;
    class CollectionsControl;
    class RenderingControl;
    class Annotation;
    struct RenderableId;

    class AnnotationsController final : public virtual IAppEventsSubscriber
    {
        public:
            AnnotationsController(AppEventsDispatcher&, IAnnotationsStore&, CollectionsControl&, RenderingControl&, SceneView&, const Icons&);
            ~AnnotationsController() override;

            bool handleAppEvent(std::type_index, const AppEvent&) override;

			bool initUserShapes();
            bool loadFromFile(const std::string&, const std::map<ShapeType, CollectionId>&);
            bool saveToFile(const std::string&, const std::map<ShapeType, CollectionId>&);

            bool init(const std::string&);
            void initAnnotation(std::uint64_t, std::uint32_t, const Annotation&);
            bool addAnnotation(const RenderableId&, OGL::Shape*, double, double);
            bool updateAnnotation(const RenderableId&, OGL::Shape*);
            bool saveAnnotation(const RenderableId&, const std::string&);

            void createAnnotationsCollection(const ShapeType&, const std::string&, const std::string&, bool);
            void createAnnotationsCollection(const ShapeType&, const std::string&, const std::string&, bool, const CollectionId&);
            bool handleDefaultCollectionVariable(const CollectionVariable&, CollectionVariable&, ShapeType, bool);

            UserGenerated* getUserGenerated();

        protected:
            AppEventsDispatcher& _appEventsDispatcher;
            UserGenerated _userGenerated;
            IAnnotationsStore& _store;
            CollectionsControl& _collectionsControl;
            RenderingControl& _renderingControl;
            SceneView& _sceneView;
            const Icons& _icons;

            std::map<CollectionId, RenderingCfg> _renderingCfgs;

            std::uint32_t _collectionIdGenerator;
            std::unordered_map<ShapeType, CollectionVariable, ShapeTypeHash> _defaultColvars;
    };
};

#endif
