// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef SCENE_VIEW_CONTROLLER_ORIGINAL_H
#define SCENE_VIEW_CONTROLLER_ORIGINAL_H

#include "tools/scene_view_controller_base.h"

namespace SEAScope
{

enum class SceneViewControllerOriginalActions : ActionType {
	CURSOR_MOVE,
	RESET,
	MOVE_START,
	MOVE,
	MOVE_END,
	ZOOM_IN_TO,
	ZOOM_IN,
	ZOOM_OUT_FROM,
	ZOOM_OUT,
	INCREASE_ZOOM_SPEED,
	DECREASE_ZOOM_SPEED,
	TOGGLE_POLES_AXIS_LOCKING,
	UPDATE_CURSOR_COORDS,
	RESIZE_VIEWPORT,
	USE_FULL_MODE,
	USE_WIREFRAME_MODE,
	USE_DOT_MODE,
};

class SceneViewControllerOriginal final : public virtual SceneViewControllerBase
{
public:
	SceneViewControllerOriginal(AppEventsDispatcher&);
	~SceneViewControllerOriginal() override;

	bool isActive() const override;
	bool handleAppEvent(std::type_index, const AppEvent&) override;

	void resetCamera() override;
	void zoomIn() override;
	void zoomOut() override;
	void lookAt(float lon, float lat) override;
	void setZoom(float altitude) override;

private:
	// Events
	void initEvents();
	void cleanEvents();
	bool performAction(const ActionType& actionType, const InputEvent& event, const InputDevicesState& devicesState) override;
	void updatePossibleActions(const std::set<SceneViewControllerOriginalActions>& toEnable, const std::set<SceneViewControllerOriginalActions>& toDisable);

	// Camera actions
	bool startCameraMovement(const InputEvent& /*event*/, const InputDevicesState& devicesState);
	bool moveCamera(const InputEvent& event, const InputDevicesState& devicesState, bool useTrackball, float direction);
	bool stopCameraMovement();
	void zoomTo(float distFromCenter);
	void zoomInTo(); // zoom on cursor

	void updateCameraRotation();
	void updateCameraZoom();
	void checkAngles();
	void cursorMoveEvent(const InputDevicesState& devicesState);
	bool cursorPosition(const glm::vec2& mousePos, glm::vec3& hitPoint);
	void setLockPolesAxis(bool lockPolesAxis);

private:
	float _zoomSpeed;
	glm::ivec2 _lastMousePosition;
	glm::vec2 _angles;
	float _zoom;
	glm::vec3 _lastGlobePos;
	glm::quat _q;

	bool _lockPolesAxis;
	glm::quat _arcBallQ;
	glm::vec3 _arcBallHitPoint;

	bool _viewLocked;
    bool _wasOnGlobe;
};

} // namespace SEAScope

#endif // SCENE_VIEW_CONTROLLER_ORIGINAL_H
