// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef NANOGUI_PATH_SELECTOR_H
#define NANOGUI_PATH_SELECTOR_H

#include <string>
#include <vector>
#include <functional>

#include "vendors/_nanogui.h"

namespace SEAScope
{
    // Forward declarations
    class NanoguiList;
    class NanoguiAnchored;
    class NanoguiTextField;
    class NanoguiClickableLabel;

    class NanoguiPathSelector: public virtual nanogui::Widget
    {
        public:
            NanoguiPathSelector(nanogui::Widget*);
            virtual ~NanoguiPathSelector() override;

            void draw(NVGcontext*) override;

            void resize(const Eigen::Vector2i&);

            void enable();
            void disable();

            void show();
            void hide();

            void setPlaceholder(const std::string&);

            const std::string& getPath() const;
            bool setPath(const std::string&);

            void toggleDropDown();
            void hideDropDown();
            void showDropDown();

            void getFullPath(const std::string&, std::string&);
            void updateExplorer(const std::string&);

            bool setChangeCallback(const std::function<void(const std::string&)>&);
            bool setDropDownCallback(const std::function<void()>&);

        protected:
            std::string _parentPath;
            NanoguiTextField* _inputText;
            NanoguiAnchored* _dropdownContainer;
            NanoguiList* _itemsList;
            nanogui::Button* _browseButton;
            nanogui::Button* _selectButton;
            nanogui::Button* _parentDirButton;
            std::vector<NanoguiClickableLabel*> _widgets;
            std::function<void(const std::string&)> _changeCallback;
            std::function<void()> _dropdownCallback;
            bool _updateRequested;
            std::string _selected;
    };
}

#endif
