// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/bbox.h"

BBox::BBox()
: BBox(glm::vec2(0.f, 0.f), glm::vec2(0.f, 0.f))
{
}

BBox::BBox(const glm::vec2& min, const glm::vec2& max)
: _min(min), _max(max)
{
}

BBox::BBox(const glm::vec4& minmax)
{
    _min = glm::vec2(minmax.x, minmax.y);
    _max = glm::vec2(minmax.z, minmax.w);
}

BBox::BBox(const glm::vec2 lonlat[4])
{
    _min = _max = lonlat[0];

    for (int i = 1; i < 4; ++i)
    {
        if (_min.x > lonlat[i].x)
        {
            _min.x = lonlat[i].x;
        }
        else if (_max.x < lonlat[i].x)
        {
            _max.x = lonlat[i].x;
        }

        if (_min.y > lonlat[i].y)
        {
            _min.y = lonlat[i].y;
        }
        else if (_max.y < lonlat[i].y)
        {
            _max.y = lonlat[i].y;
        }
    }

    _min = glm::radians(_min);
    _max = glm::radians(_max);
}
