// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/buffer_desc.h"

#include "core/opengl_enum.h"

namespace
{
    unsigned int getDataTypeSize(OGL::DataType dataType)
    {
        unsigned int size = 0;

        switch(dataType)
        {
        case OGL::unsigned_byte:
            size = 1;
            break;
        case OGL::unsigned_short:
            size = 2;
            break;
        case OGL::unsigned_int:
        case OGL::float32:
            size = 4;
            break;
        case OGL::none:
        case OGL::DataTypeCount:
            size = 0;
            break;
        }

        return size;
    }
}

BufferDesc::BufferDesc()
:   _length(0),
    _componentCount(0),
    _dataType(OGL::none)
{
}

BufferDesc::BufferDesc(unsigned int length, unsigned int componentCount, OGL::DataType dataType)
:   _length(length),
    _componentCount(componentCount),
    _dataType(dataType)
{
}

unsigned int BufferDesc::getComponentCount() const
{
    return _componentCount;
}

OGL::DataType BufferDesc::getDataType() const
{
    return _dataType;
}

unsigned int BufferDesc::getLength() const
{
    return _length;
}

unsigned int BufferDesc::getArraySize() const
{
    return getLength() * getElementSize();
}

unsigned int BufferDesc::getComponentSize() const
{
    return getDataTypeSize(getDataType());
}

unsigned int BufferDesc::getElementSize() const
{
    return getComponentCount() * getComponentSize();
}
