// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/icons.h"
#include "3d/texture.h"
#include "3d/texture_loader.h"

Icons::Icons()
{
}

Icons::~Icons()
{
    for
        (auto& it: this->_textures)
    {
        delete it.second;
        it.second = nullptr;
    }
    this->_textures.clear();
}

void Icons::setTexture(IconType iconType, OGL::Texture* texture)
{
    auto found = this->_textures.find(iconType);
    if
        (this->_textures.end() != found)
    {
        if
            (found->second != texture)
        {
            delete found->second;
            found->second = nullptr;
        }
    }

    if
        (nullptr != texture)
    {
        this->_textures[iconType] = texture;
    }
}

void Icons::setTexture(IconType iconType, OGL::Context* ctx, const std::string& texturePath)
{
    OGL::Texture* texture = TextureLoader::loadFromFile(ctx, texturePath);
    this->setTexture(iconType, texture);
}

OGL::Texture* Icons::getTexture(IconType iconType, bool& ok) const
{
    auto found = this->_textures.find(iconType);
    if
        (this->_textures.end() == found)
    {
        ok = false;
        return  nullptr;
    }

    ok = true;
    return found->second;
}
