// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "3d/texture_loader.h"

// File reader
#include "3d/ppm.h"
#include "3d/png_reader.h"

// OpenGL
#include "3d/texture.h"
#include "3d/context.h"
#include "core/opengl_enum.h"
#include "core/log.h"

OGL::Texture* TextureLoader::loadFromFile(OGL::Context* ctx, const std::string& filePath)
{
    int width(0), height(0);
    unsigned char* buffer(nullptr);

    std::string file_name = filePath;

    size_t pos = file_name.rfind('.');
    if (std::string::npos == pos)
    {
        return nullptr;
    }

    std::string ext = file_name.substr(pos + 1, file_name.length() - (pos + 1));

    bool error = false;
    int channels(0);

    if (ext == "png")
    {
        error = readPNG(filePath.c_str(), width, height, channels, buffer);
    }
    else if (ext == "ppm")
    {
        error = loadPPM(filePath.c_str(), width, height, buffer);
        channels = 3;
    }

    if (false == error)
    {
        SEAScope::Log::log(SEAScope::LogLevel::Error, "TextureLoader",
                           "An error occurred during texture loading");
        return nullptr;
    }

    OGL::Texture* tex = ctx->createTexture(OGL::texture2D);
    tex->create(ctx);
    tex->allocate2D(width, height, channels, OGL::unsigned_byte, buffer);

    free(buffer);

    return tex;
}
