// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <cstring>

#include "vendors/_glm.h"
#include "core/copy.h"

void memcpy2D(void * src,
              unsigned src_width,
              unsigned src_height,
              unsigned int element_size,
              void * dst,
              unsigned int origin_x,
              unsigned int origin_y,
              unsigned dst_width,
              unsigned dst_height)
{
    if (nullptr == src || nullptr == dst)
    {
        throw;
    }

    if (origin_x + src_width > dst_width)
    {
        throw;
    }

    if (origin_y + src_height > dst_height)
    {
        throw;
    }

    unsigned int src_row_size = src_width * element_size;
    unsigned int dst_row_size = dst_width * element_size;

    char* copy_origin = static_cast<char*>(dst) + ((origin_y * dst_width + origin_x) * element_size);

    for (unsigned int y = 0; y < src_height; ++y)
    {
        std::memcpy(copy_origin + dst_row_size * y, static_cast<char*>(src) + src_row_size * y, src_row_size);
    }
}
