// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <sstream>
#include "core/settings.h"


using namespace SEAScope;


/**
 * Constructor.
 */
Settings::Settings(): logFilter("")
{

};

/**
 * Destructor.
 */
Settings::~Settings()
{

};

/**
 * Initialize with default settings.
 */
bool Settings::init(const std::string& configPath)
{
    size_t found;               //
    std::string configDir;      //
	std::ostringstream buffer;  //

    found = configPath.find_last_of("/\\");
    configDir = configPath.substr(0, found);

	buffer << configDir << "/plugins";
	this->pluginsPath = buffer.str();
	buffer.str("");

	buffer << configDir << "/colormaps";
	this->colormapsPath = buffer.str();
	buffer.str("");

	buffer << configDir << "/operators";
	this->operatorsPath = buffer.str();
	buffer.str("");

	buffer << configDir << "/data";
	this->dataPath = buffer.str();
	buffer.str("");

	this->glRenderer = "OpenGL3";
	this->glContextMajor = 3;
	this->glContextMinor = 3;
	this->windowWidth = 800;
	this->windowHeight = 800;
	this->vsync = false;
	this->antialiasing = true;
	this->antialiasingType = "msaa";
	this->antialiasingFactor = 8;
	this->fullscreen = false;
	this->vsync = true;
	this->defaultColormap = "grayscale";
	this->baseLayer = "none";
	this->worldMap = "earth.png";
	this->skipIndexCheck = false;
	this->timelineCfg.minYear = 1980;
	this->timelineCfg.maxYear = 2020;
	this->timelineCfg.defaultTimespan = "";
	this->timelineCfg.defaultTimestep = "";
    this->timelineCfg.useNearestMode = true;
    this->timelineCfg.useNearestInDayMode = true;
	this->minGranuleScreenEstate = 0.01f;
	this->processorEnabled = false;
	this->processorAddress = "127.0.0.1";
	this->processorPort = 53450;
    this->searchRequestMinDelay = 0.1;

	return true;
};

/**
 * Modify a setting.
 */
template <typename T>
bool Settings::set(const SettingsKey, const T /*value*/)
{
	return false;
};

/**
 * Retrieve a setting.
 */
template <typename T>
const T Settings::get(const SettingsKey) const
{
	return SettingsValue::NOT_FOUND;
};
