// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "core/tools.h"
#include "core/app_events_dispatcher.h"

using namespace SEAScope;

/**
 *
 */
Tools::Tools(AppEventsDispatcher& appEventsDispatcher)
: _windowController(appEventsDispatcher),
  _sceneViewController(appEventsDispatcher),
  _extractionController(appEventsDispatcher),
  _transectController(appEventsDispatcher),
  _selectionController(appEventsDispatcher)
{
    this->_windowController.setId(0);
    this->_sceneViewController.setId(1);
    this->_debugController.setId(2);
    this->_debugController.setSceneViewController(&(this->_sceneViewController));
    this->_extractionController.setId(3);
    this->_transectController.setId(4);
    this->_selectionController.setId(5);
}

/**
 *
 */
Tools::~Tools()
{

}

/**
 *
 */
WindowController* Tools::getWindowController()
{
    return &(this->_windowController);
}

/**
 *
 */
SceneViewControllerBase* Tools::getSceneViewController()
{
    return &(this->_sceneViewController);
}

/**
 *
 */
DebugController* Tools::getDebugController()
{
    return &(this->_debugController);
}

/**
 *
 */
ExtractionController* Tools::getExtractionController()
{
    return &(this->_extractionController);
}

/**
 *
 */
TransectController* Tools::getTransectController()
{
    return &(this->_transectController);
}

/**
 *
 */
SelectionController* Tools::getSelectionController()
{
    return &(this->_selectionController);
}
