// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "3d/stdafx.h"
#include "core/workspace_folders.h"
#include "core/path.h"
#include "core/log.h"

WorkspaceFolders::WorkspaceFolders()
{
}

WorkspaceFolders::~WorkspaceFolders()
{

}

bool WorkspaceFolders::setWorkspacePath(const std::string& path)
{
	this->_workspacePath = path;

	if (false == SEAScope::Path::dirExists(this->_workspacePath))
	{
		SEAScope::Log::log(SEAScope::LogLevel::Error, "WorkspaceFolders", "Workspace path doesn't exist : {}", path);
		return false;
	}

	if
		('/' != *this->_workspacePath.rbegin() &&
		'\\' != *this->_workspacePath.rbegin())
	{
		this->_workspacePath += SEAScope::Path::separator();
	}

	this->_imagePath = this->_workspacePath + "images" + SEAScope::Path::separator();

	if (false == SEAScope::Path::dirExists(this->_imagePath))
	{
		SEAScope::Log::log(SEAScope::LogLevel::Error, "WorkspaceFolders", "images folder doesn't exist : {}", this->_imagePath);
		return false;
	}

	this->_iconPath = this->_imagePath + "icons" + SEAScope::Path::separator();

	if (false == SEAScope::Path::dirExists(this->_iconPath))
	{
		SEAScope::Log::log(SEAScope::LogLevel::Error, "WorkspaceFolders", "icons folder doesn't exist : {}", this->_iconPath);
		return false;
	}

	this->_barbsPath = this->_imagePath + "windBarbs" + SEAScope::Path::separator() + "pgm" + SEAScope::Path::separator();

	if (false == SEAScope::Path::dirExists(this->_barbsPath))
	{
		SEAScope::Log::log(SEAScope::LogLevel::Error, "WorkspaceFolders", "barbs folder doesn't exist : {}", this->_barbsPath);
		return false;
	}

	this->_arrowsPath = this->_imagePath + "arrows" + SEAScope::Path::separator() + "pgm" + SEAScope::Path::separator();

	if (false == SEAScope::Path::dirExists(this->_arrowsPath))
	{
		SEAScope::Log::log(SEAScope::LogLevel::Error, "WorkspaceFolders", "barbs folder doesn't exist : {}", this->_arrowsPath);
		return false;
	}

	this->_shaderPath = this->_workspacePath + "GLSL" + SEAScope::Path::separator();

	if (false == SEAScope::Path::dirExists(this->_shaderPath))
	{
		SEAScope::Log::log(SEAScope::LogLevel::Error, "WorkspaceFolders", "GLSL folder doesn't exist : {}", this->_shaderPath);
		return false;
	}

	// Optional file
	this->_styleFilePath = this->_workspacePath + "style.ini";

	if (false == SEAScope::Path::fileExists(this->_styleFilePath))
	{
		SEAScope::Log::log(SEAScope::LogLevel::Info, "WorkspaceFolders", "style.ini file doesn't exist : {}", this->_styleFilePath);
		this->_styleFilePath = "";
	}

	return true;
}

const std::string& WorkspaceFolders::getImageFolderPath() const
{
	return this->_imagePath;
}

const std::string& WorkspaceFolders::getIconFolderPath() const
{
	return this->_iconPath;
}

const std::string& WorkspaceFolders::getShaderFolderPath() const
{
	return this->_shaderPath;
}

const std::string& WorkspaceFolders::getbBarbsFolderPath() const
{
	return this->_barbsPath;
}

const std::string& WorkspaceFolders::getbArrowsFolderPath() const
{
	return this->_arrowsPath;
}

const std::string& WorkspaceFolders::getStyleFilePath() const
{
	return this->_styleFilePath;
}
