// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "core/log.h"
#include "dummy_metadata_store.h"

using namespace SEAScope;

/**
 *
 */
DummyMetadataStore::DummyMetadataStore()
{
};

/**
 *
 */
DummyMetadataStore::~DummyMetadataStore()
{

};

/**
 *
 */
bool DummyMetadataStore::listCollections(std::set<CollectionId>& result) const
{
	for
		(const auto& it : this->collections)
	{
		result.insert(it.first);
	}

	return true;
};

/**
 *
 */
bool DummyMetadataStore::getCollectionCfg(const CollectionId& collectionId, CollectionCfg& cfg) const
{
	auto found = this->collections.find(collectionId);
	if
		(this->collections.end() == found)
	{
		return false;
	}
	cfg = found->second;
    return true;
};

/**
 *
 */
bool DummyMetadataStore::setCollectionCfg(const CollectionId& collectionId, const CollectionCfg& cfg)
{
    auto found = this->collections.find(collectionId);
    if
        (this->collections.end() == found)
    {
	    this->collections[collectionId] = cfg;
        return true;
    }
    return false;
};

/**
 *
 */
bool DummyMetadataStore::addVariable(const CollectionId& collectionId, const VariableId& variableId, const VariableCfg& vCfg)
{
	auto found = this->collections.find(collectionId);
	if
		(this->collections.end() == found)
	{
		Log::log( LogLevel::Error, "DummyMetadataStore"
				, "Could not add variable to a non-registered collection: {}/{}:{}"
				, collectionId.first, collectionId.second, variableId);
		return false;
	}

	auto varFound = found->second.variables.find(variableId);
	if
		(found->second.variables.end() != varFound)
	{
		Log::log( LogLevel::Error, "DummyMetadataStore"
				, "Cannot add variable if it already exists");
		return false;
	}

	found->second.variables[variableId] = vCfg;

	if
		(1 == found->second.variables.size())
	{
		found->second.defaultVariable = variableId;
	}

	return true;
};

/**
 *
 */
bool DummyMetadataStore::deleteCollectionCfg(const CollectionId& collectionId)
{
    auto found = this->collections.find(collectionId);
    if
        (this->collections.end() == found)
    {
        return false;
    }

    this->collections.erase(found);
    return true;
}

/**
 *
 */
bool DummyMetadataStore::removeVariable(const CollectionId& collectionId, const VariableId& variableId)
{
	auto found = this->collections.find(collectionId);
	if
		(this->collections.end() == found)
	{
		Log::log( LogLevel::Error, "DummyMetadataStore"
				, "Could not remove variable to a non-registered collection: {}/{}:{}"
				, collectionId.first, collectionId.second, variableId);
		return false;
	}

	auto varFound = found->second.variables.find(variableId);
	if
		(found->second.variables.end() == varFound)
	{
		Log::log( LogLevel::Error, "DummyMetadataStore"
				, "Cannot remove variable if it was not registered in the first place");
		return false;
	}

    found->second.variables.erase(varFound);
    return true;
}
