// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "events/transect_request.h"

#include "3d/scene_view.h"
#include "3d/shape_mesh.h"
#include "core/structs.h"

using namespace SEAScope;

TransectRequest::TransectRequest(SceneView* sceneView, ShapeMesh* footprint, std::map<RenderableId, Extraction>* extractedData, std::map<RenderableId, std::string>* extractedLabels)
: _sceneView(sceneView), _footprint(footprint), _extractedData(extractedData), _extractedLabels(extractedLabels)
{

}

TransectRequest::~TransectRequest()
{
    this->_sceneView = nullptr; // not owning
    this->_footprint = nullptr; // not owning
    this->_extractedData = nullptr; // not owning
    this->_extractedLabels = nullptr;   // not owning
}

SceneView* TransectRequest::getSceneView() const
{
    return this->_sceneView;
}

ShapeMesh* TransectRequest::getFootprint() const
{
    return this->_footprint;
}

std::map<RenderableId, Extraction>* TransectRequest::getExtractedData() const
{
    return this->_extractedData;
}

std::map<RenderableId, std::string>* TransectRequest::getExtractedLabels() const
{
    return this->_extractedLabels;
}
