// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "vendors/_nanogui.h"

#include "ui/nanogui/nanogui_about_menu.h"
#include "ui/nanogui/nanogui_selectable_label.h"
#include "core/log.h"

#if defined __linux__ || defined __APPLE__
	#include "seascope_version.h"
#endif

using namespace SEAScope;

/**
 *
 */
NanoguiAboutMenu::NanoguiAboutMenu():
window(nullptr), title(nullptr), closeBtn(nullptr), closeCallback(nullptr)
{
};

/**
 *
 */
NanoguiAboutMenu::~NanoguiAboutMenu()
{
};

/**
 *
 */
bool NanoguiAboutMenu::init(nanogui::Widget* screen)
{
	nanogui::Widget* headerbar;	//
	std::int32_t headerHeight;	//

	if
		(nullptr != this->window)
	{
		/* Already initialized */
		return false;
	}

	headerHeight = 20;

	this->window = new NanoguiDraggablePane(screen);
	this->window->headerHeight = static_cast<std::uint32_t>(headerHeight);
    this->window->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                                   nanogui::Alignment::Middle,
                                                   0, 0));

	headerbar = new nanogui::Widget(this->window);
	headerbar->setFixedSize(Eigen::Vector2i(360, headerHeight));
	headerbar->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal
											   , nanogui::Alignment::Middle
											   , 2, 0));

	this->title = new nanogui::Label(headerbar, "About SEAScope", "sans-bold");
	this->title->setFixedWidth(340);

	this->closeBtn = new nanogui::Button(headerbar, "", ENTYPO_ICON_CROSS);
	this->closeBtn->setFixedSize(Eigen::Vector2i(16, 16));
	this->closeBtn->setCallback([this]() {
		if
			(nullptr != this->closeCallback)
		{
			this->closeCallback();
		}
	});

    this->window->keyCallback = [this] (int key, int /*scancode*/, int action, int /*modifiers*/) -> bool {
        if
            (action == GLFW_PRESS || action == GLFW_REPEAT)
        {
            if
                (GLFW_KEY_ESCAPE == key
                && nullptr != this->closeCallback)
            {
                this->closeCallback();
                return true;
            }
        }

        return false;
    };

    nanogui::Widget* block;
    block = new nanogui::Widget(this->window);
    block->setLayout(new nanogui::BoxLayout( nanogui::Orientation::Vertical
                                           , nanogui::Alignment::Middle
                                           , 5, 0));

    std::ostringstream buffer("");
    buffer << "SEAScope is a 3D visualisation and analysis application for\n";
    buffer << "satellite, in-situ and numerical model data.\n\n";
    buffer << "Copyright (C) 2014-2022 OceanDataLab\n";
    buffer << "License LGPLv3+\n\n";
    buffer << "New features and improvements have been sponsored by BPI FRANCE and ESA SEOM\n\n";

    nanogui::Label* lbl = new nanogui::Label(block, buffer.str(), "sans-bold");
    lbl->setFixedSize(Eigen::Vector2i(340, 136));

    lbl = new nanogui::Label(block, "Getting help", "sans-bold");
    lbl->setFontSize(20);

    new nanogui::Label(block, "Please consult documentation on the SEAScope website.", "sans-bold");
    NanoguiSelectableLabel* tbx = new NanoguiSelectableLabel(block);
    tbx->setValue("https://seascope.oceandatalab.com");
    tbx->setFixedSize(Eigen::Vector2i(320,30));
    tbx->setAlignment(nanogui::TextBox::Alignment::Center);
    new nanogui::Label(block, "\nDo not forget to look at the FAQ and tutorials,", "sans-bold");
    new nanogui::Label(block, "they may provide some useful information :)\n", "sans-bold");

    lbl = new nanogui::Label(block, "\nYou can also ask questions on the forum:", "sans-bold");
    lbl->setFixedSize(Eigen::Vector2i(320, 30));

    tbx = new NanoguiSelectableLabel(block);
    tbx->setValue("https://forum.oceandatalab.com/seascope");
    tbx->setFixedSize(Eigen::Vector2i(320,30));
    tbx->setAlignment(nanogui::TextBox::Alignment::Center);

    lbl = new nanogui::Label(block, "\nIf your question cannot be disclosed publicly, you can contact us be email at:", "sans-bold");
    lbl->setFixedSize(Eigen::Vector2i(320, 50));

    tbx = new NanoguiSelectableLabel(block);
    tbx->setValue("seascope@oceandatalab.com");
    tbx->setFixedSize(Eigen::Vector2i(320,30));
    tbx->setAlignment(nanogui::TextBox::Alignment::Center);


    #if defined __linux__ || defined __APPLE__
    lbl = new nanogui::Label(block, "\nBuild information", "sans-bold");
    lbl->setFontSize(20);
    lbl->setFixedHeight(30);

    buffer.str("");
    for
        (const auto& it : SEAScope::getVersion())
    {
        if
            (  "date" != it.first
            && "ref" != it.first
            && "glm_version" != it.first
            && "glibc_version" != it.first
            && "stdc++_version" != it.first)
        {
            continue;
        }
        buffer << it.first << ": " << it.second << "\n";
    }
    lbl = new nanogui::Label(block, buffer.str(), "sans-bold");
    lbl->setFixedWidth(320);
    #endif

    return true;
};

/**
 *
 */
bool NanoguiAboutMenu::display(const Eigen::Vector2i& cursorPos)
{
    bool ok;    //
    Eigen::Vector2i pos;    //

    ok = this->adaptPosition(cursorPos, pos);
    ok = this->window->display(pos);
    return ok;
};

/**
 *
 */
bool NanoguiAboutMenu::adaptPosition(const Eigen::Vector2i& cursorPos, Eigen::Vector2i& result) const
{
    bool fitFound;  //
    bool fits;  //
    std::int32_t x;   //
    std::int32_t y;   //
    double xTmp;    //
    double yTmp;    //

    fitFound = true;
    const Eigen::Vector2i& parentSize = this->window->parent()->size();
    const Eigen::Vector2i& selfSize = this->window->size();

    /* By default, center on the cursor horizontal position */
    xTmp = std::floor(cursorPos.x() - 0.5 * selfSize.x());
    x = static_cast<std::int32_t>(xTmp);
    if
        (0 > x)
    {
        /* Move the menu to the right of the cursor with a small margin */
        x = cursorPos.x() + 20;
        if
            (parentSize.x() < x + selfSize.x())
        {
            /* the screen estate is too small, move menu at the horizontal
               center of the screen */
            xTmp = std::floor(0.5 * (parentSize.x() - selfSize.x()));
            x = static_cast<std::int32_t>(xTmp);
            fits = (0<x && parentSize.x() < (x + selfSize.x()));
            fitFound = fitFound && fits;
        }
    }

    if
        (parentSize.x() < x + selfSize.x())
    {
        /* Move the menu to the left of the cursor with a small margin */
        x = cursorPos.x() - selfSize.x() - 20;
        if
            (0 > x)
        {
            /* the screen estate is too small, move menu at the horizontal
               center of the screen */
            xTmp = std::floor(0.5 * (parentSize.x() - selfSize.x()));
            x = static_cast<std::int32_t>(xTmp);
            fits = (0<x && parentSize.x() < (x + selfSize.x()));
            fitFound = fitFound && fits;
        }
    }

    /* The menu is displayed below the cursor by default */
    y = cursorPos.y() + 20;
    if
        (parentSize.y() < y + selfSize.y())
    {
        /* Move the menu above cursor */
        y = cursorPos.y() - 20 - selfSize.y();
        if
            (0 > y)
        {
            yTmp = std::floor(0.5 * (parentSize.y() - selfSize.y()));
            y = static_cast<std::int32_t>(yTmp);
            fits = (0<y && parentSize.y() < (y + selfSize.y()));
            fitFound = fitFound && fits;
        }
    }

    result = Eigen::Vector2i(x,y);
    return fitFound;
};

/**
 *
 */
bool NanoguiAboutMenu::setTarget()
{
    this->title->setCaption("Help");

    this->window->updateLayout();
    return true;
};
