// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <math.h>
#include <iomanip>

#include "vendors/_nanogui.h"
#include "ui/nanogui/nanogui_numberbox.h"

#include "core/log.h"
#include "ui/nanogui/nanogui_advanced_rendering_control.h"

using namespace SEAScope;

/**
 *
 */
NanoguiAdvancedRenderingControl::NanoguiAdvancedRenderingControl(nanogui::Widget* parent)
: nanogui::Widget(parent),
 _scaleCallback(nullptr),
 _cmapCallback(nullptr),
 _colorCallback(nullptr),
 _resetCallback(nullptr),
 _filterModeCallback(nullptr),
 _activationCallback(nullptr),
 _streamlinesCallback(nullptr),
 _cmapManager(nullptr),
 _label(""),
 _labelBuffer(""),
 _resetButton(nullptr),
 _nearestBtn(nullptr),
 _bilinearBtn(nullptr),
 _minField(nullptr),
 _maxField(nullptr),
 _logScale(nullptr),
 _palette(nullptr),
 _scale(nullptr),
 _colorbar(nullptr),
 _tick1(nullptr),
 _tick2(nullptr),
 _tick3(nullptr),
 _tick4(nullptr),
 _helper(nullptr),
 _colorChooser(nullptr),
 _cursorY(0),
 _cursorTxt(""),
 _reflowNeeded(false),
 _leftSide(nullptr),
 _rightSide(nullptr),
 _filteringWidget(nullptr),
 _colorscaleWidget(nullptr),
 _streamlinesWidget(nullptr),
 _particlesField(nullptr),
 _lengthField(nullptr),
 _speedField(nullptr),
 _leftSpacer(nullptr)
{
}

/**
 *
 */
NanoguiAdvancedRenderingControl::~NanoguiAdvancedRenderingControl()
{
};

/**
 *
 */
void NanoguiAdvancedRenderingControl::resize(const Eigen::Vector2i& size)
{
    if
        (nullptr != this->_leftSide)
    {
        this->_leftSide->setFixedSize(Eigen::Vector2i(250, size.y() - 6));
        this->_leftSide->setSize(Eigen::Vector2i(250, size.y() - 6));
    }

    if
        (nullptr != this->_rightSide)
    {
        this->_rightSide->setFixedSize(Eigen::Vector2i(116, size.y() - 6));
        this->_rightSide->setSize(Eigen::Vector2i(116, size.y() - 6));
    }

	if
		(nullptr != this->_palette)
	{
		this->_palette->setFixedSize(Eigen::Vector2i(size.x(), size.y() - 43));
		this->_palette->setSize(Eigen::Vector2i(size.x(), size.y() - 43));
	}
	if
		(nullptr != this->_scale)
	{
		this->_scale->setFixedSize(Eigen::Vector2i(size.x() - 42, size.y() - 43));
		this->_scale->setSize(Eigen::Vector2i(size.x() - 42, size.y() - 43));
	}

    /*
	Log::log( LogLevel::Debug, "NanoguiAdvancedRenderingControl"
			, "setSize: {} {} {} {}"
			, this->palette->fixedWidth()
			, this->palette->fixedHeight()
			, this->scale->fixedWidth()
			, this->scale->fixedHeight());
    */

	return nanogui::Widget::setFixedSize(size);
};

/**
 *
 */
void NanoguiAdvancedRenderingControl::show()
{
	nanogui::Widget::setVisible(true);
    if
        (nullptr != this->_activationCallback)
    {
        this->_activationCallback(true);
    }
}

/**
 *
 */
void NanoguiAdvancedRenderingControl::hide()
{
    if
        (nullptr != this->_helper)
    {
        this->_helper->setVisible(false);
    }
	nanogui::Widget::setVisible(false);
    if
        (nullptr != this->_activationCallback)
    {
        this->_activationCallback(false);
    }
};

/**
 *
 */
void NanoguiAdvancedRenderingControl::buildPalette(nanogui::Widget* parent)
{
	this->_palette = new nanogui::Widget(parent);
	this->_palette->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
												     nanogui::Alignment::Middle,
												     0, 0));

	/* numbers */
	this->_scale = new nanogui::Widget(this->_palette);
	this->_scale->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
												   nanogui::Alignment::Maximum,
												   0, 31));

	this->_maxField = new NanoguiNumberBox(this->_scale);
	this->_maxField->setEditable(true);
	this->_maxField->setFontSize(14);
	this->_maxField->setFixedSize(Eigen::Vector2i(70, 20));
	this->_maxField->setCallback([this](float) {
		if
			(nullptr != this->_scaleCallback)
		{
			this->_scaleCallback(static_cast<float>(this->_minField->value()),
							     static_cast<float>(this->_maxField->value()),
							     this->_logScale->checked());
		}
	});

	this->_tick1 = new nanogui::Label(this->_scale, "");
	this->_tick1->setFontSize(14);
	this->_tick1->setFixedHeight(20);
	this->_tick2 = new nanogui::Label(this->_scale, "");
	this->_tick2->setFontSize(14);
	this->_tick2->setFixedHeight(20);
	this->_tick3 = new nanogui::Label(this->_scale, "");
	this->_tick3->setFontSize(14);
	this->_tick3->setFixedHeight(20);
	this->_tick4 = new nanogui::Label(this->_scale, "");
	this->_tick4->setFontSize(14);
	this->_tick4->setFixedHeight(20);

	this->_minField = new NanoguiNumberBox(this->_scale);
	this->_minField->setEditable(true);
	this->_minField->setFontSize(14);
	this->_minField->setFixedSize(Eigen::Vector2i(70, 20));
	this->_minField->setCallback([this](float) {
		if
			(nullptr != this->_scaleCallback)
		{
			this->_scaleCallback(static_cast<float>(this->_minField->value()),
							     static_cast<float>(this->_maxField->value()),
							     this->_logScale->checked());
		}
	});

    nanogui::Widget* spacer = new nanogui::Widget(this->_palette);
    spacer->setFixedWidth(5);

	/* colors */
	this->_helper = NanoguiAnchored::create(parent);
	this->_helper->placement = NanoguiAnchored::ON_RIGHT;
	this->_helper->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
												    nanogui::Alignment::Middle,
												    3, 5));
	this->_helper->setVisible(false);
	new nanogui::Label(this->_helper, "Color selection", "sans-bold");
	this->_colorChooser = new NanoguiColorbarList(this->_helper);
	this->_colorChooser->setFixedHeight(250);
    this->_colorChooser->colormaps->setFixedHeight(150);
	this->_colorChooser->init(this->_cmapManager->getColormaps());
	this->_colorChooser->setColormapCallback([this](const std::string& labelIn, const Colormap& /*cmap*/) {
		if
			(nullptr != this->_cmapCallback)
		{
			this->_cmapCallback(labelIn);
		}
	});
	this->_colorChooser->setColorCallback([this](const std::array<std::uint8_t, 3>& color) {
		if
			(nullptr != this->_colorCallback)
		{
			this->_colorCallback(color);
		}
	});

	this->_colorbar = new NanoguiHColorbar(this->_palette);
	this->_colorbar->setFixedSize(Eigen::Vector2i(20,255));
	this->_colorbar->orientation = nanogui::Orientation::Vertical;

}

/**
 *
 */
void NanoguiAdvancedRenderingControl::buildSettingsPanel(nanogui::Widget* parent)
{
    std::vector<nanogui::Button*> filterBtns(2);   //
    nanogui::Widget* spacer(nullptr);   //
    nanogui::Label* lbl(nullptr);
    nanogui::Widget* filteringWidget(nullptr);  //
    nanogui::Widget* filterBtnsContainer(nullptr);  //
    nanogui::Widget* colorscaleContainer(nullptr);  //
    nanogui::Widget* streamlinesContainer(nullptr); //
    nanogui::Widget* particlesContainer(nullptr);   //
    nanogui::Widget* lengthContainer(nullptr);  //
    nanogui::Widget* speedContainer(nullptr);   //
    nanogui::Widget* resetContainer(nullptr);   //

    spacer = new nanogui::Widget(parent);
    spacer->setFixedSize(Eigen::Vector2i(105, 0));

    // Filtering
    this->_filteringWidget = new nanogui::Widget(parent);
    this->_filteringWidget->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                                             nanogui::Alignment::Minimum,
                                                             0, 0));
    new nanogui::Label(this->_filteringWidget, "Filtering", "sans-bold");

    filteringWidget = new nanogui::Widget(this->_filteringWidget);
    filteringWidget->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                                      nanogui::Alignment::Minimum,
                                                      0, 0));
    spacer = new nanogui::Widget(filteringWidget);
    spacer->setFixedWidth(5);

    filterBtnsContainer = new nanogui::Widget(filteringWidget);
    filterBtnsContainer->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                                          nanogui::Alignment::Minimum,
                                                          0, 1));

    filterBtns[0] = new nanogui::Button(filterBtnsContainer, "Nearest");
    filterBtns[1] = new nanogui::Button(filterBtnsContainer, "Bilinear");
    this->_nearestBtn = filterBtns[0];
    this->_bilinearBtn = filterBtns[1];

    this->_nearestBtn->setButtonGroup(filterBtns);
    this->_nearestBtn->setFontSize(14);
    this->_nearestBtn->setFlags(nanogui::Button::RadioButton);
	this->_nearestBtn->setFixedSize(Eigen::Vector2i(80, 18));
    this->_nearestBtn->setTooltip("Nearest value");
    this->_nearestBtn->setCallback([this] () {
        if
            (nullptr != this->_filterModeCallback)
        {
            this->_filterModeCallback("nearest");
        }
    });

    this->_bilinearBtn->setButtonGroup(filterBtns);
    this->_bilinearBtn->setFontSize(14);
    this->_bilinearBtn->setFlags(nanogui::Button::RadioButton);
	this->_bilinearBtn->setFixedSize(Eigen::Vector2i(80, 18));
    this->_bilinearBtn->setTooltip("Bilinear interpolation");
    this->_bilinearBtn->setCallback([this] () {
        if
            (nullptr != this->_filterModeCallback)
        {
            this->_filterModeCallback("bilinear");
        }
    });

    spacer = new nanogui::Widget(parent);
    spacer->setFixedHeight(3);

    // Colorscale
    this->_colorscaleWidget = new nanogui::Widget(parent);
    this->_colorscaleWidget->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                                              nanogui::Alignment::Minimum,
                                                              0, 0));
    new nanogui::Label(this->_colorscaleWidget, "Color scale", "sans-bold");

    colorscaleContainer = new nanogui::Widget(this->_colorscaleWidget);
    colorscaleContainer->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                                          nanogui::Alignment::Middle,
                                                          0, 0));
    spacer = new nanogui::Widget(colorscaleContainer);
    spacer->setFixedWidth(5);

	this->_logScale = new NanoguiCheckbox(colorscaleContainer, "Logarithmic");
	this->_logScale->setTooltip("Use a logarithmic scale");
	this->_logScale->setFixedSize(Eigen::Vector2i(100, 16));
	this->_logScale->setCallback([this](bool) {
		if
			(nullptr != this->_scaleCallback)
		{
			this->_scaleCallback(static_cast<float>(this->_minField->value()),
							     static_cast<float>(this->_maxField->value()),
							     this->_logScale->checked());
		}
	});

    spacer = new nanogui::Widget(parent);
    spacer->setFixedHeight(3);

    this->_streamlinesWidget = new nanogui::Widget(parent);
    this->_streamlinesWidget->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                                              nanogui::Alignment::Minimum,
                                                              0, 0));
    // Streamlines
    new nanogui::Label(this->_streamlinesWidget, "Streamlines", "sans-bold");

    streamlinesContainer = new nanogui::Widget(this->_streamlinesWidget);
    streamlinesContainer->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                                           nanogui::Alignment::Middle,
                                                           0, 0));
    // - particles
    particlesContainer = new nanogui::Widget(streamlinesContainer);
    particlesContainer->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                                         nanogui::Alignment::Middle,
                                                         0, 0));
    spacer = new nanogui::Widget(particlesContainer);
    spacer->setFixedWidth(5);
    lbl = new nanogui::Label(particlesContainer, "Particles");
    lbl->setFixedSize(Eigen::Vector2i(50, 20));
    this->_particlesField = new NanoguiNumberBox(particlesContainer);
    this->_particlesField->setRequiresKeyboardValidation(true);
    this->_particlesField->setValidationFramesDelay(40);
    this->_particlesField->setValidMin(1000);
    this->_particlesField->setDelta(1000);
    this->_particlesField->setEditable(true);
	this->_particlesField->setFontSize(14);
	this->_particlesField->setFixedSize(Eigen::Vector2i(50, 20));
    this->_particlesField->setCallback([this] (float value) {
        if
            (value < static_cast<float>(this->_particlesField->getDelta()))
        {
            value = static_cast<float>(this->_particlesField->getDelta());
            this->_particlesField->setValue(this->_particlesField->getDelta());
        }

        if
            (nullptr != this->_streamlinesCallback)
        {
            this->_streamlinesCallback(static_cast<std::uint32_t>(value),
                                       static_cast<std::uint32_t>(this->_lengthField->value()),
                                       static_cast<float>(this->_speedField->value()));
        }
    });

    // - length
    lengthContainer = new nanogui::Widget(streamlinesContainer);
    lengthContainer->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                                      nanogui::Alignment::Middle,
                                                      0, 0));
    spacer = new nanogui::Widget(lengthContainer);
    spacer->setFixedWidth(5);
    lbl = new nanogui::Label(lengthContainer, "Length");
    lbl->setFixedSize(Eigen::Vector2i(50, 20));
    this->_lengthField = new NanoguiNumberBox(lengthContainer);
    this->_lengthField->setRequiresKeyboardValidation(true);
    this->_lengthField->setValidationFramesDelay(40);
    this->_lengthField->setValidMin(1);
    this->_lengthField->setDelta(1);
    this->_lengthField->setEditable(true);
	this->_lengthField->setFontSize(14);
	this->_lengthField->setFixedSize(Eigen::Vector2i(50, 20));
    this->_lengthField->setCallback([this] (float value) {
        if
            (value < static_cast<float>(this->_lengthField->getDelta()))
        {
            value = static_cast<float>(this->_lengthField->getDelta());
            this->_lengthField->setValue(this->_lengthField->getDelta());
        }

        if
            (nullptr != this->_streamlinesCallback)
        {
            this->_streamlinesCallback(static_cast<std::uint32_t>(this->_particlesField->value()),
                                       static_cast<std::uint32_t>(value),
                                       static_cast<float>(this->_speedField->value()));
        }
    });

    // - speed
    speedContainer = new nanogui::Widget(streamlinesContainer);
    speedContainer->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                                     nanogui::Alignment::Middle,
                                                     0, 0));
    spacer = new nanogui::Widget(speedContainer);
    spacer->setFixedWidth(5);
    lbl = new nanogui::Label(speedContainer, "Speed");
    lbl->setFixedSize(Eigen::Vector2i(50, 20));
    this->_speedField = new NanoguiNumberBox(speedContainer);
    this->_speedField->setRequiresKeyboardValidation(true);
    this->_speedField->setValidationFramesDelay(40);
    this->_speedField->setValidMin(0.1);
    this->_speedField->setDelta(0.1);
    this->_speedField->setEditable(true);
    this->_speedField->setFontSize(14);
    this->_speedField->setFixedSize(Eigen::Vector2i(50, 20));
    this->_speedField->setCallback([this] (float value) {
        if
            (value < static_cast<float>(this->_speedField->getDelta()))
        {
            value = static_cast<float>(this->_speedField->getDelta());
            this->_speedField->setValue(this->_speedField->getDelta());
        }

        if
            (nullptr != this->_streamlinesCallback)
        {
            this->_streamlinesCallback(static_cast<std::uint32_t>(this->_particlesField->value()),
                                       static_cast<std::uint32_t>(this->_lengthField->value()),
                                       value);
        }
    });

    spacer = new nanogui::Widget(parent);
    spacer->setFixedHeight(3);

    // Reset rendering settings
    resetContainer = new nanogui::Widget(parent);
    resetContainer->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                                     nanogui::Alignment::Middle,
                                                     0, 0));

	this->_saveButton = new nanogui::Button(resetContainer, "Save", ENTYPO_ICON_SAVE);
	this->_saveButton->setFixedSize(Eigen::Vector2i(70, 20));
	this->_saveButton->setTooltip("Save current configuration in the config.ini");
	this->_saveButton->setCallback([this]() {
		Log::log(LogLevel::Debug, "NanoguiAdvancedRendering", "Save current configuration");
		if
			(nullptr != this->_saveCallback)
		{
			this->_saveCallback();
		}
	});

	this->_resetButton = new nanogui::Button(resetContainer, "Reset", ENTYPO_ICON_BACK);
	this->_resetButton->setFixedSize(Eigen::Vector2i(70, 20));
	this->_resetButton->setTooltip("Restore original configuration");
	this->_resetButton->setCallback([this](){
		Log::log(LogLevel::Debug, "NanoguiAdvancedRendering", "Reset rendering configuration");
		if
			(nullptr != this->_resetCallback)
		{
			this->_resetCallback();
		}
	});
}

/**
 *
 */
bool NanoguiAdvancedRenderingControl::init(const ColormapManager* cmapManagerIn)
{
	nanogui::Widget* spacer(nullptr);	//

	this->_cmapManager = cmapManagerIn;


    this->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Horizontal,
                                           nanogui::Alignment::Middle,
                                           0, 3));

	/* Add an empty space for the caption */
	spacer = new nanogui::Label(this, "");
	spacer->setFixedWidth(24);

    this->_leftSide = new nanogui::Widget(this);
    this->_leftSide->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
                                                      nanogui::Alignment::Fill,
                                                      0, 3));
    this->buildSettingsPanel(this->_leftSide);

    this->_leftSpacer = new nanogui::Widget(this);
    this->_leftSpacer->setFixedSize(Eigen::Vector2i(67, 0));
    this->_leftSpacer->setVisible(false);

    this->_rightSide = new nanogui::Widget(this);
	this->_rightSide->setLayout(new nanogui::BoxLayout(nanogui::Orientation::Vertical,
										               nanogui::Alignment::Fill,
										               0, 0));

    this->buildPalette(this->_rightSide);

	this->_colorbar->setClickCallback([this](const std::string& /*label*/, const Colormap& /*cmap*/, int buttons, float ratio, bool targetIsMax) {
		if
			(true == this->_helper->visible())
		{
				this->_helper->setVisible(false);
		}
		else if
			(1 /* right button */ == buttons)
		{
			Log::log(LogLevel::Debug, "NanoguiPaletteControl", "Toggle colormaps list");
			this->_helper->setVisible(! this->_helper->visible());
		}
		else
		{
			this->changeExtremum(targetIsMax, ratio);
		}
	});
	this->_colorbar->setMouseMoveCallback([this](bool /*targetsMax*/, float ratio) {
        double ratioTmp(0.0);   //
		double newValue(0.0);	//
        double newYTmp(0.0);    //
        double colorbarHeightTmp(0.0);  //
		int newY(0);	//
		std::ostringstream buffer("");	//

		if
			(true == this->_helper->visible())
		{
			return;
		}

        ratioTmp = static_cast<double>(ratio);

		if
			(false == this->_logScale->checked())
		{
			newValue = this->_minField->value() + ratioTmp * (this->_maxField->value() - this->_minField->value());
		}
		else if
			(  0 < this->_minField->value()
			&& 0 < this->_maxField->value())
		{
			newValue = exp(log(this->_minField->value()) + ratioTmp * (log(this->_maxField->value()) - log(this->_minField->value())));
			newValue = fmax(0.0001, newValue);
		}
		else
		{
			Log::log(LogLevel::Error, "NanoguiPaletteControl", "Invalid scale: log scale can not be used with negative values");
			return;
		}
		buffer << std::fixed << std::setprecision(3) << newValue;

        colorbarHeightTmp = static_cast<double>(this->_colorbar->height());
		newYTmp = std::ceil(colorbarHeightTmp * ratioTmp);
        newY = static_cast<int>(newYTmp);
        newY = this->_colorbar->height() - newY;
		newY = std::min(newY, this->_colorbar->height());
		newY = std::max(newY, 0);
		newY += this->_colorbar->position().y();

		this->_cursorTxt = buffer.str();
		this->_cursorY = static_cast<uint16_t>(newY);
	});
	this->_colorbar->setDragEndCallback([this](bool targetIsMax, float ratio) {
		if
			(true == this->_helper->visible())
		{
			return;
		}

		this->changeExtremum(targetIsMax, ratio);
	});

    this->_reflowNeeded = true;

	return true;
};

/**
 *
 */
void NanoguiAdvancedRenderingControl::changeExtremum(bool targetIsMax, float ratio)
{
	double newValueTmp(0.0);	//
    double ratioTmp(0.0);    //
    float newValue(0.f); //
    float minTmp(0.f);  //
    float maxTmp(0.f);  //

	if
		(nullptr != this->_scaleCallback)
	{
        ratioTmp = static_cast<double>(ratio);

		if
			(false == this->_logScale->checked())
		{
			newValueTmp = this->_minField->value() + ratioTmp * (this->_maxField->value() - this->_minField->value());
		}
		else if
			(  0 < this->_minField->value()
			&& 0 < this->_maxField->value())
		{
			newValueTmp = exp(log(this->_minField->value()) + ratioTmp * (log(this->_maxField->value()) - log(this->_minField->value())));
			newValueTmp = fmax(0.0001, newValueTmp);
		}
		else
		{
			Log::log(LogLevel::Error, "NanoguiPaletteControl", "Invalid scale: log scale can not be used with negative values");
			this->_minField->setUnits("");
			this->_maxField->setUnits("");
			return;
		}

        newValue = static_cast<float>(newValueTmp);

		if
			(true == targetIsMax)
		{
            minTmp = static_cast<float>(this->_minField->value());
			this->_scaleCallback(minTmp,
							     newValue,
							     this->_logScale->checked());
		}
		else
		{
            maxTmp = static_cast<float>(this->_maxField->value());
			this->_scaleCallback(newValue,
							     maxTmp,
							     this->_logScale->checked());
		}
	}

	this->_minField->setUnits("");
	this->_maxField->setUnits("");
};

/**
 *
 */
void NanoguiAdvancedRenderingControl::draw(NVGcontext* ctx)
{
	float labelOffset;	//
	float labelWidth;	//
    float posX; //
    float posY; //
    float sizeX;    //
    float sizeY;    //
    float rightX;   //
    float rightY;   //
    float colorbarPosX; //
    float colorbarPosY; //
    float colorbarSizeX;    //

	if
		(false == this->mVisible)
	{
		return;
	}

    if
        (nullptr == this->_helper)
    {
        // Component has not been initialized yet
        return;
    }

    posX = static_cast<float>(this->mPos.x());
    posY = static_cast<float>(this->mPos.y());
    sizeX = static_cast<float>(this->mSize.x());
    sizeY = static_cast<float>(this->mSize.y());
    colorbarPosX = static_cast<float>(this->_colorbar->position().x());
    colorbarPosY = static_cast<float>(this->_colorbar->position().y());
    colorbarSizeX = static_cast<float>(this->_colorbar->width());
    rightX = static_cast<float>(this->_rightSide->position().x());
    rightY = static_cast<float>(this->_rightSide->position().y());

	if
		(true == this->_reflowNeeded)
	{
		this->performLayout(ctx);
		this->_reflowNeeded = false;
	}

	/* draw background */
	nvgBeginPath(ctx);
	nvgRoundedRect( ctx
                  , posX + 1.0f, posY + 1.0f
				  , sizeX - 2.0f, sizeY - 1.0f
				  , 0);
	nvgFillColor(ctx, nanogui::Color(50,50,50,170));
	nvgFill(ctx);

	/*Draw caption background*/
	nvgBeginPath(ctx);
	nvgRoundedRect( ctx
                  , posX + 1.0f, posY + 1.0f
				  , 24.0f, sizeY - 1.0f
				  , 0);
	nvgFillColor(ctx, nanogui::Color(50,50,50,128));
	nvgFill(ctx);

	if
		(  false == this->_helper->visible()
		&& true == this->_colorbar->dragging)
	{
		if
			(true == this->_colorbar->dragTargetIsMax)
		{
			this->_maxField->setUnits(" *");
		}
		else
		{
			this->_minField->setUnits(" *");
		}
	}
	nanogui::Widget::draw(ctx);

    /* Draw vertical label */
	nvgSave(ctx);
	nvgTranslate(ctx, posX + 10.0f, posY + sizeY);
	nvgRotate(ctx, nvgDegToRad(-90));
	nvgFontFace(ctx, "sans");
	nvgFontSize(ctx, 14);
	nvgFillColor(ctx, nanogui::Color(255,255,255,255));
	nvgTextAlign(ctx, NVG_ALIGN_LEFT | NVG_ALIGN_MIDDLE);
    nvgText(ctx, 4, 0, this->_labelBuffer.c_str(), nullptr);
	nvgRestore(ctx);

    /*Draw tick marks*/
    if
        (true == this->_minField->visible())
    {
        nvgBeginPath(ctx);
        nvgMoveTo(ctx,
                  posX + rightX + colorbarPosX - 5.0f,
                  posY + rightY + colorbarPosY + 10.0f + 31.0f + 10.0f);
        nvgLineTo(ctx,
                  posX + rightX + colorbarPosX,
                  posY + rightY + colorbarPosY + 10.0f + 31.0f + 10.0f);
        nvgStrokeColor(ctx, nanogui::Color(235,235,235,200));
        nvgStroke(ctx);

        nvgBeginPath(ctx);
        nvgMoveTo(ctx,
                  posX + rightX + colorbarPosX - 5.0f,
                  posY + rightY + colorbarPosY + 10.0f + 31.0f + 20.0f + 31.0f + 10.0f);
        nvgLineTo(ctx,
                  posX + rightX + colorbarPosX,
                  posY + rightY + colorbarPosY + 10.0f + 31.0f + 20.0f + 31.0f + 10.0f);
        nvgStrokeColor(ctx, nanogui::Color(235,235,235,200));
        nvgStroke(ctx);

        nvgBeginPath(ctx);
        nvgMoveTo(ctx,
                  posX + rightX + colorbarPosX - 5.0f,
                  posY + rightY + colorbarPosY + 10.0f + 31.0f + 20.0f + 31.0f + 20.0f + 31.0f + 10.0f);
        nvgLineTo(ctx,
                  posX + rightX + colorbarPosX,
                  posY + rightY + colorbarPosY + 10.0f + 31.0f + 20.0f + 31.0f + 20.0f + 31.0f + 10.0f);
        nvgStrokeColor(ctx, nanogui::Color(235,235,235,200));
        nvgStroke(ctx);

        nvgBeginPath(ctx);
        nvgMoveTo(ctx,
                  posX + rightX + colorbarPosX - 5.0f,
                  posY + rightY + colorbarPosY + 10.0f + 31.0f + 20.0f + 31.0f + 20.0f + 31.0f + 20.0f + 31.0f + 10.0f);
        nvgLineTo(ctx,
                  posX + rightX + colorbarPosX,
                  posY + rightY + colorbarPosY + 10.0f + 31.0f + 20.0f + 31.0f + 20.0f + 31.0f + 20.0f + 31.0f + 10.0f);
        nvgStrokeColor(ctx, nanogui::Color(235,235,235,200));
        nvgStroke(ctx);
    }

	/* draw cursor */
	if
		(  false == this->_helper->visible()
        && true == this->_minField->visible()
		&& (  true == this->_colorbar->isMouseOver()
		   || true == this->_colorbar->dragging))
	{
		nvgBeginPath(ctx);
		nvgMoveTo(ctx, posX + rightX + colorbarPosX - 3.0f, posY + rightY + static_cast<float>(this->_cursorY) + 1.0f);
		nvgLineTo(ctx, posX + rightX + colorbarPosX + 3.0f, posY + rightY + static_cast<float>(this->_cursorY) + 1.0f);
		nvgStrokeColor(ctx, nanogui::Color(0,0,0,200));
		nvgStroke(ctx);

		nvgBeginPath(ctx);
		nvgMoveTo(ctx, posX + rightX + colorbarPosX + colorbarSizeX - 3.0f, posY + rightY + static_cast<float>(this->_cursorY) + 1.0f);
		nvgLineTo(ctx, posX + rightX + colorbarPosX + colorbarSizeX + 3.0f, posY + rightY + static_cast<float>(this->_cursorY) + 1.0f);
		nvgStrokeColor(ctx, nanogui::Color(0,0,0,200));
		nvgStroke(ctx);

		nvgMoveTo(ctx, 0, 0);

		nvgBeginPath(ctx);
		nvgFontFace(ctx, "sans-bold");
		nvgFontSize(ctx, 14);
		labelWidth = nvgTextBounds( ctx
								  , 0
								  , 0
								  , this->_cursorTxt.c_str()
								  , nullptr, nullptr);

		labelOffset = posX + rightX + colorbarPosX - 16.0f - labelWidth;
		nvgMoveTo(ctx
                 , labelOffset - 8.0f
				 , posY + rightY + static_cast<float>(this->_cursorY) - 8.0f + 1.0f);
		nvgLineTo( ctx
                 , labelOffset + labelWidth + 4.0f
				 , posY + rightY + static_cast<float>(this->_cursorY) - 8.0f + 1.0f);
		nvgLineTo( ctx
                 , labelOffset + labelWidth + 12.0f
				 , posY + rightY + static_cast<float>(this->_cursorY) + 1.0f);
		nvgLineTo( ctx
                 , labelOffset + labelWidth + 4.0f
				 , posY + rightY + static_cast<float>(this->_cursorY) + 8.0f + 1.0f);
		nvgLineTo( ctx
                 , labelOffset + labelWidth + 4.0f
				 , posY + rightY + static_cast<float>(this->_cursorY) + 8.0f + 1.0f);
		nvgLineTo( ctx
                 , labelOffset - 8.0f
				 , posY + rightY + static_cast<float>(this->_cursorY) + 8.0f + 1.0f);
		nvgLineTo(ctx
                 , labelOffset - 8.0f
				 , posY + rightY + static_cast<float>(this->_cursorY) - 8.0f + 1.0f);

		nvgFillColor(ctx, nanogui::Color(235, 235, 235, 200));
		nvgFill(ctx);
		if
			(this->_colorbar->dragging)
		{
			nvgStrokeColor(ctx, nanogui::Color(255,50,30,230));
			nvgStroke(ctx);
		}

		nvgFillColor(ctx, nanogui::Color(30,30,30,255));
		nvgTextAlign(ctx, NVG_ALIGN_LEFT | NVG_ALIGN_MIDDLE);
		nvgText( ctx, labelOffset - 2.0f
			   , posY + rightY + static_cast<float>(this->_cursorY) + 1.0f, this->_cursorTxt.c_str(), nullptr);
	}
};

/**
 *
 */
bool NanoguiAdvancedRenderingControl::setScaleChangedCallback(const std::function<void(float,float,bool)>& callback)
{
	this->_scaleCallback = callback;
	return true;
};

/**
 *
 */
bool NanoguiAdvancedRenderingControl::setColormapChangedCallback(const std::function<void(const std::string&)>& callback)
{
	this->_cmapCallback = callback;
	return true;
};

/**
 *
 */
bool NanoguiAdvancedRenderingControl::setColorChangedCallback(const std::function<void(const std::array<std::uint8_t, 3>&)>& callback)
{
	this->_colorCallback = callback;
	return true;
};

bool NanoguiAdvancedRenderingControl::update(RenderingCfg* cfg)
{
	bool ok;	//
	ok = this->update(cfg, this->_label);
	return ok;
};

/**
 *
 */
bool NanoguiAdvancedRenderingControl::update(RenderingCfg* cfg, const std::string& labelIn)
{
	float extent;	//
	float step;	//
	std::ostringstream buffer;  //
    double extentTmp;   //
	double delta;
	bool ok;	//
	Colormap cmap;	//
    float labelWidth(0.f);  //
    float heightTmp(0.f);   //
    double labelSize(0.0);  //
    std::string labelBufferTmp(labelIn);    //

	/* update label */
	this->_label = labelIn;

    heightTmp = static_cast<float>(this->fixedHeight());

    // Remove 7 chars to insert ellipsis
    std::size_t removeCharsStart = 3;
    std::size_t removeCharsEnd = 4;
    labelSize = static_cast<double>(labelIn.size());
    double splitStartTmp = std::floor(0.5 * labelSize);
    double splitStopTmp = std::ceil(0.5 * labelSize);
    std::size_t splitStart = static_cast<std::size_t>(splitStartTmp);
    std::size_t splitStop = static_cast<std::size_t>(splitStopTmp);

    nvgSave(this->screen()->nvgContext());
	nvgFontFace(this->screen()->nvgContext(), "sans");
	nvgFontSize(this->screen()->nvgContext(), 14);
    labelWidth = nvgTextBounds( this->screen()->nvgContext()
                              , 0
                              , 0
                              , labelBufferTmp.c_str()
                              , nullptr, nullptr);

    while
        (0 < labelBufferTmp.size() && labelWidth > heightTmp - 4.0f)
    {
        std::ostringstream tmpBuffer;
        tmpBuffer << labelIn.substr(0, splitStart - removeCharsStart);
        tmpBuffer << " [...] ";
        tmpBuffer << labelIn.substr(splitStop + removeCharsEnd, labelIn.size());
        labelBufferTmp = tmpBuffer.str();

        if
            (removeCharsStart > removeCharsEnd)
        {
            removeCharsEnd = removeCharsEnd + 1;
        }
        else
        {
            removeCharsStart = removeCharsStart + 1;
        }

        labelWidth = nvgTextBounds( this->screen()->nvgContext()
                                  , 0
                                  , 0
                                  , labelBufferTmp.c_str()
                                  , nullptr, nullptr);
    }
    nvgRestore(this->screen()->nvgContext());

    this->_labelBuffer = labelBufferTmp;

	/* Update colormap */
	this->_colorbar->cmap.clear();
	ok = this->_cmapManager->getColormap(cfg->colormap(), cmap);
	if
		(true == ok)
	{
		this->_colorbar->setColormap(cmap);
		this->_colorbar->label = cfg->colormap();

	}
	else
	{
		this->_colorbar->setColor(cfg->color());
	}
    this->_filteringWidget->setVisible(ok);
    this->_colorscaleWidget->setVisible(ok);
    this->_minField->setVisible(ok);
    this->_tick1->setVisible(ok);
    this->_tick2->setVisible(ok);
    this->_tick3->setVisible(ok);
    this->_tick4->setVisible(ok);
    this->_maxField->setVisible(ok);
    this->_leftSpacer->setVisible(!ok);

	this->_colorChooser->colorwheel->setColor(nanogui::Color(cfg->color()[0],
														     cfg->color()[1],
														     cfg->color()[2],
														     255));

	/* Update scale */
	if
		(false ==  cfg->logscale())
	{
		extent = cfg->max() - cfg->min();
        extentTmp = static_cast<double>(extent);
		delta = 0.02 * extentTmp;
	}
	else if
		(0 < cfg->min() && cfg->min() < cfg->max())
	{
		extent = log(cfg->max()) - log(cfg->min());
        extentTmp = static_cast<double>(extent);
		delta = exp(0.02 * extentTmp);
	}
	else
	{
		Log::log( LogLevel::Debug, "NanoguiAdvancedRenderingControl"
				, "Log scale can not be used with range [{}, {}]"
				, cfg->min(), cfg->max());
		return false;
	}

	step = extent * 0.2f;

	Log::log(LogLevel::Debug, "NanoguiAdvancedRendering", " **** **** **** UPDATE DELTA: {} {} {} **** **** ****"
			, extent, step, 0.1f * step);

    double minTmp = static_cast<double>(cfg->min());
	this->_minField->setValue(minTmp);
	this->_minField->setDelta(delta);
	if
		(false == cfg->logscale())
	{
		buffer << std::fixed << std::setprecision(3) << (cfg->min() + 1.0f * step) << " ";
		this->_tick4->setCaption(buffer.str());
		buffer.str("");
		buffer << std::fixed << std::setprecision(3) << (cfg->min() + 2.0f * step) << " ";
		this->_tick3->setCaption(buffer.str());
		buffer.str("");
		buffer << std::fixed << std::setprecision(3) << (cfg->min() + 3.0f * step) << " ";
		this->_tick2->setCaption(buffer.str());
		buffer.str("");
		buffer << std::fixed << std::setprecision(3) << (cfg->min() + 4.0f * step) << " ";
		this->_tick1->setCaption(buffer.str());
		buffer.str("");
	}
	else
	{
		buffer << std::fixed << std::setprecision(3) << exp(log(cfg->min()) + 1.0f * step) << " ";
		this->_tick4->setCaption(buffer.str());
		buffer.str("");
		buffer << std::fixed << std::setprecision(3) << exp(log(cfg->min()) + 2.0f * step) << " ";
		this->_tick3->setCaption(buffer.str());
		buffer.str("");
		buffer << std::fixed << std::setprecision(3) << exp(log(cfg->min()) + 3.0f * step) << " ";
		this->_tick2->setCaption(buffer.str());
		buffer.str("");
		buffer << std::fixed << std::setprecision(3) << exp(log(cfg->min()) + 4.0f * step) << " ";
		this->_tick1->setCaption(buffer.str());
		buffer.str("");
	}

    this->_nearestBtn->setPushed(FilterMode::NEAREST == cfg->filterMode());
    this->_bilinearBtn->setPushed(FilterMode::BILINEAR == cfg->filterMode());

    double maxTmp = static_cast<double>(cfg->max());
	this->_maxField->setValue(maxTmp);
	this->_maxField->setDelta(delta);
	this->_logScale->setChecked(cfg->logscale());

    /* Streamlines */
    if
        (RenderMethod::STREAMLINES == cfg->renderMethod())
    {
        this->_particlesField->setValue(cfg->particlesCount());
        this->_lengthField->setValue(cfg->streamlineLength());
        this->_speedField->setValue(static_cast<double>(cfg->streamlineSpeed()));

        this->_streamlinesWidget->setVisible(true);
        this->_filteringWidget->setVisible(false);
        this->_colorChooser->colormaps->setVisible(true);
    }
    else if
        (  RenderMethod::POINT == cfg->renderMethod()
        || RenderMethod::POLYLINE == cfg->renderMethod()
        || RenderMethod::POLYGON == cfg->renderMethod())
    {
        this->_streamlinesWidget->setVisible(false);
        this->_colorChooser->colormaps->setVisible(false);
    }
    else
    {
        this->_streamlinesWidget->setVisible(false);
        this->_colorChooser->colormaps->setVisible(true);
    }

	// sourceId = 1 python collection
	// sourceId = 2 annotation
	// sourceId = 3+ "normal" collection, local data
	if
		(3 > cfg->target.collection.first)
	{
		this->_saveButton->setVisible(false);
	}
	else
	{
		this->_saveButton->setVisible(true);
	}

    this->_reflowNeeded = true;

	return true;
};

/**
 *
 */
bool NanoguiAdvancedRenderingControl::setResetCallback(const std::function<void()>& callback)
{
	this->_resetCallback = callback;
	return true;
};

/**
 *
 */
bool NanoguiAdvancedRenderingControl::setFilterModeCallback(const std::function<void(const std::string&)>& callback)
{
    this->_filterModeCallback = callback;
    return true;
};

/**
 *
 */
bool NanoguiAdvancedRenderingControl::scrollEvent(const Eigen::Vector2i& p, const Eigen::Vector2f& rel)
{
	nanogui::Widget::scrollEvent(p, rel);
	return true;
};

/**
 *
 */
bool NanoguiAdvancedRenderingControl::setActivationCallback(const std::function<void(bool activated)>& callback)
{
    this->_activationCallback = callback;
    return true;
}

/**
 *
 */
bool NanoguiAdvancedRenderingControl::setStreamlinesCallback(const std::function<void(std::uint32_t, std::uint32_t, float)>& callback)
{
    this->_streamlinesCallback = callback;
    return true;
}

bool NanoguiAdvancedRenderingControl::setSaveConfigCallback(const std::function<void()>& callback)
{
	this->_saveCallback = callback;
	return true;
}
