// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include "core/log.h"
#include "ui/nanogui/nanogui_renderer.h"
#include "glfw_window.h"


using namespace SEAScope;

/**
 *
 */
NanoguiRenderer::NanoguiRenderer()
{
};

/**
 *
 */
NanoguiRenderer::~NanoguiRenderer()
{

};

/**
 *
 */
bool NanoguiRenderer::setWindow(IWindow& window)
{
	GLFWWindow* w;	//

	w = dynamic_cast<GLFWWindow*>(&window);
	if
		(nullptr == w)
	{
		return false;
	}
	this->initialize(w->implementation, false);
	return true;
};

/**
 *
 */
bool NanoguiRenderer::renderFrame()
{
	if
		(false == this->visible())
	{
		return false;
	}

	this->drawWidgets();

	return true;
};

/**
 *
 */
bool NanoguiRenderer::handleMouseMoveEvent(double x, double y)
{
	return this->cursorPosCallbackEvent(x,y);
};

/**
 *
 */
bool NanoguiRenderer::handleKeyEvent(int key, int scancode, int action, int mods)
{
	return this->keyCallbackEvent(key, scancode, action, mods);
};

/**
 *
 */
bool NanoguiRenderer::handleCharEvent(unsigned int c, int /*mods*/)
{
	return this->charCallbackEvent(c);
};

/**
 *
 */
bool NanoguiRenderer::handleMouseButtonEvent(int button, int action, int mods)
{
	return this->mouseButtonCallbackEvent(button, action, mods);
};

/**
 *
 */
bool NanoguiRenderer::handleMouseWheelEvent(double x, double y)
{
	return this->scrollCallbackEvent(x,y);
};

/**
 *
 */
bool NanoguiRenderer::handleDropEvent(int count, const char** filenames)
{
	return this->dropCallbackEvent(count, filenames);
};

/**
 *
 */
bool NanoguiRenderer::handleResizeEvent(int width, int height)
{
	bool ok;	//

	ok = this->resizeCallbackEvent(width, height);
	if
		(nullptr != this->resizeCallback)
	{
		this->resizeCallback(width, height);
	}
	return ok;
};
