// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <iostream>
#include <cstdlib>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <GL/gl.h>
#include <GL/glx.h>

int main(int /*argc*/, const char** /*argv*/){
		Display *dpy;
		Window root;
		GLint attr[] = { GLX_RGBA, GLX_DEPTH_SIZE, 24, GLX_DOUBLEBUFFER, None };
		XVisualInfo *vi;
		GLXContext glc;

		/* open display */
		if ( ! (dpy = XOpenDisplay(nullptr)) ) {
				std::cerr << "cannot connect to X server" << std::endl;
				exit(1);
		}

		/* get root window */
		root = DefaultRootWindow(dpy);

		/* get visual matching attr */
		if( ! (vi = glXChooseVisual(dpy, 0, attr)) ) {
				std::cerr << "no appropriate visual found" << std::endl;
				exit(1);
		}

		/* create a context using the root window */
		if ( ! (glc = glXCreateContext(dpy, vi, nullptr, GL_TRUE)) ){
				std::cerr << "failed to create context" << std::endl;
				exit(1);
		}
		glXMakeCurrent(dpy, root, glc);

		/* try it out, remember to *NOT* render to the default framebuffer! */
		std::cout << glGetString(GL_VERSION) << std::endl;

		return 0;
}
