:: =============================================================================
:: @author: <guillaume.leseach@oceandatalab.com>
:: @date: 2019-06-28
:: 
:: This file is part of SEAScope, a 3D visualisation and analysis application
:: for satellite, in-situ and numerical model data.
::
:: Copyright (C) 2014-2022 OceanDataLab
::
:: SEAScope is free software: you can redistribute it and/or modify it under
:: the terms of the GNU Lesser General Public License as published by the Free
:: Software Foundation, either version 3 of the License, or (at your option)
:: any later version.
::
:: SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
:: WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
:: FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
:: details.
::
:: You should have received a copy of the GNU Lesser General Public License
:: along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
:: =============================================================================
@echo off 
setlocal
SET _devdir=%~dp0
SET _devdir=%_devdir:~0,-1%
for %%i in ("%_devdir%") do set _projectdir=%%~dpi
SET _clientdir=%_projectdir:~0,-1%
SET _pluginsdir=%_clientdir%\..\embedded-server

SET _compilationTarget=Release

if "%1" == "debug" (
	SET _compilationTarget=Debug
)

SET PLUGINS_WORKSPACE=%_pluginsdir%\thirdparty
SET WORKSPACE=%_clientdir%\thirdparty
SET _bindir=%WORKSPACE%\bin\%_compilationTarget%
SET _exedir=%WORKSPACE%\bin\exe
SET _libdir=%WORKSPACE%\lib\%_compilationTarget%
SET _seascopebindir=%_clientdir%\bin\%_compilationTarget%

mkdir %WORKSPACE%
cd %WORKSPACE%
mkdir bin
mkdir %_bindir%
mkdir bin\exe
mkdir lib
mkdir lib\%_compilationTarget%
mkdir tmp

cd %_clientdir%
mkdir bin
mkdir bin\%_compilationTarget%

SET _fmtdir=%WORKSPACE%\fmt
SET _glfwdir=%WORKSPACE%\glfw
SET _glmdir=%WORKSPACE%\glm
SET _cxxoptsdir=%WORKSPACE%\cxxopts
SET _zlibdir=%WORKSPACE%\zlib
SET _pngdir=%WORKSPACE%\lpng
SET _netcdfdir=%WORKSPACE%\netCDF
SET _netcdfcxxdir=%WORKSPACE%\netcdf-cxx4
SET _sqlitedir=%WORKSPACE%\sqlite
SET _nanoguidir=%WORKSPACE%\nanogui
SET _flatbuffersdir=%WORKSPACE%\flatbuffers
SET _tmpdir=%WORKSPACE%\tmp
SET _picojsondir=%WORKSPACE%\picojson

if %_compilationTarget% == Release (
	SET _fmtlib=fmt.lib
	SET _zlib=zlibstatic.lib
	SET _pnglib=libpng16.lib
	SET _pngdll=libpng16.dll
) else (
	if %_compilationTarget% == Debug (
		SET _fmtlib=fmtd.lib
		SET _zlib=zlibstaticd.lib
		SET _pnglib=libpng16d.lib
		SET _pngdll=libpng16d.dll
	)
)

SET _glfwlib=glfw3.lib
SET _sqlitelib=sqlite3.lib
SET _nanoguilib=nanogui-obj.lib
SET _flatcexe=flatc.exe

SET _fmturl=https://github.com/fmtlib/fmt.git
SET _glfwurl=https://github.com/glfw/glfw/releases/download/3.3.6/glfw-3.3.6.bin.WIN64.zip
SET _glmurl=https://github.com/g-truc/glm.git
SET _cxxoptsurl=https://github.com/jarro2783/cxxopts.git
SET _gluurl=https://anongit.freedesktop.org/git/mesa/glu.git
SET _iniparserurl=https://github.com/Poordeveloper/ini-parser.git
SET _zliburl=https://www.zlib.net/zlib1212.zip
SET _lpngurl=https://downloads.sourceforge.net/project/libpng/libpng16/1.6.37/lpng1637.zip?r=https%3A%2F%2Fsourceforge.net%2Fprojects%2Flibpng%2Ffiles%2Flibpng16%2F1.6.37%2Flpng1637.zip%2Fdownload%3Fuse_mirror%3Dnetcologne%26download%3D
SET _sqliteurl=https://www.sqlite.org/2019/sqlite-amalgamation-3280000.zip
SET _nanoguiurl=https://github.com/wjakob/nanogui.git
SET _flatbuffersurl=https://github.com/google/flatbuffers.git
SET _picojsonurl=https://github.com/kazuho/picojson.git

::
::
::	Downloads
::
::

REM FMT
cd %WORKSPACE%
if not exist %_fmtdir% (
	git clone %_fmturl% %_fmtdir%
	cd %_fmtdir%
	git checkout d561cb47a76ea0563721809dfbd4f717741f21ca
	cmake -G "Visual Studio 17 2022" -A x64 .
)

REM GLFW
cd %WORKSPACE%
if not exist %_glfwdir% (
	if not exist %_tmpdir%\glfw.zip (
		powershell -Command "(New-Object Net.WebClient).DownloadFile('%_glfwurl%', '%_tmpdir%\glfw.zip')"
	)
	powershell.exe -NoP -NonI -Command "Expand-Archive -Path '%_tmpdir%\glfw.zip' -DestinationPath '%WORKSPACE%'"
	rename glfw-3.3.6.bin.WIN64 glfw
	copy glfw\lib-vc2022\glfw3.lib %_libdir%
)

REM GLM
cd %WORKSPACE%
if not exist %_glmdir% (
	git clone "%_glmurl%" glm
	cd glm
	git checkout 0.9.9.0
)

REM CXXOPTS
cd %WORKSPACE%
if not exist %_cxxoptsdir% (
	git clone "%_cxxoptsurl%" cxxopts
	cd cxxopts
	git checkout 3876c0907237e5fa89c5850ed1ee688a3bcb62b3
)

REM GLU
cd %WORKSPACE%
if not exist glu (
	git clone %_gluurl% glu
)

REM INI-PARSER
cd %WORKSPACE%
if not exist ini-parser (
	git clone %_iniparserurl% ini-parser
)

REM ZLIB
cd %WORKSPACE%
if not exist %_zlibdir% (
	if not exist %_tmpdir%\zlib.zip (
		powershell -Command "(New-Object Net.WebClient).DownloadFile('%_zliburl%', '%_tmpdir%\zlib.zip')"
	)
	powershell.exe -NoP -NonI -Command "Expand-Archive -Path '%_tmpdir%\zlib.zip' -DestinationPath '%WORKSPACE%'"
	rename zlib-1.2.12 zlib
	cd %_zlibdir%
	cmake -G "Visual Studio 17 2022" -A x64 .
)

REM LIBPNG
cd %WORKSPACE%
if not exist %_pngdir% (
	if not exist %_tmpdir%\libpng.zip (
		powershell -Command "(New-Object Net.WebClient).DownloadFile('%_lpngurl%', '%_tmpdir%\libpng.zip')"
	)
	powershell.exe -NoP -NonI -Command "Expand-Archive '%_tmpdir%\libpng.zip' -DestinationPath '%WORKSPACE%'"
	rename lpng1637 lpng
	cd lpng
	cmake -G "Visual Studio 17 2022" -A x64 -DZLIB_INCLUDE_DIR=%_zlibdir% -DZLIB_LIBRARY=%_libdir%\%_zlib% .
)

REM SQLITE
cd %WORKSPACE%
if not exist %_sqlitedir% (
	if not exist %_tmpdir%\sqlite.zip (
		powershell -Command "(New-Object Net.WebClient).DownloadFile('%_sqliteurl%', '%_tmpdir%\sqlite.zip')"
	)
	powershell.exe -NoP -NonI -Command "Expand-Archive '%_tmpdir%\sqlite.zip'  -DestinationPath '%WORKSPACE%'"
	rename sqlite-amalgamation-3280000 sqlite
)

REM NANOGUI
cd %WORKSPACE%
if not exist %_nanoguidir% (
	git clone %_nanoguiurl% --recursive %_nanoguidir%
	cd %_nanoguidir%
    git checkout 3e09f4f0b75a00da0e8ac4164f0d90cb600e0251
	cmake -G "Visual Studio 17 2022" -A x64 .
)

REM FLATBUFFERS
cd %WORKSPACE%
if not exist %_flatbuffersdir% (
	git clone %_flatbuffersurl% %_flatbuffersdir%
	cd %_flatbuffersdir%
	git checkout 63cc0eec4e59015c91ab09894cb6822191a28151
	cmake -G "Visual Studio 17 2022" -A x64 .
	powershell -Command "(Get-Content flatc.vcxproj) | Foreach-Object {$_ -replace '<TreatWarningAsError>true</TreatWarningAsError>','<TreatWarningAsError>false</TreatWarningAsError>'} | Out-File flatc.vcxproj"
)

REM PICOJSON
cd %WORKSPACE%
if not exist %_picojsondir% (
	git clone %_picojsonurl% %_picojsondir%
	cd %_picojsondir%
	git checkout 8ba7113451b745b6e4bd83db12f73eefbb315d2e
)

REM
REM
REM	Compilation
REM
REM


REM FMT
if not exist %_libdir%\%_fmtlib% (
	cd %_fmtdir%
	msbuild fmt.vcxproj /property:Configuration=%_compilationTarget%
	copy %_compilationTarget%\%_fmtlib% %_libdir%
)

REM GLFW
if not exist %_libdir%\%_glfwlib% (
	cd %WORKSPACE%
	copy %_glfwdir%\lib-vc2019\%_glfwlib% %_libdir%
)

REM GLM

REM CXXOPTS

REM GLU

REM INI-PARSER

REM ZLIB
if not exist %_libdir%\%_zlib% (
	cd %_zlibdir%
	msbuild zlibstatic.vcxproj /property:Configuration=%_compilationTarget%
	copy %_compilationTarget%\%_zlib% %_libdir%
)

REM LIBPNG
if not exist %_libdir%\%_pnglib% (
	cd %_pngdir%
	msbuild png.vcxproj  /property:Configuration=%_compilationTarget%
	copy %_compilationTarget%\%_pnglib% %_libdir%
	copy %_compilationTarget%\%_pngdll% %_bindir%
)

REM SQLITE
if not exist %_libdir%\%_sqlitelib% (
	cd %_sqlitedir%
	cl /c /EHsc sqlite3.c
	cl /c /EHsc shell.c
	lib sqlite3.obj shell.obj /OUT:%_sqlitelib%
	copy %_sqlitelib% %_libdir%
)

REM NANOGUI
if not exist %_libdir%\%_nanoguilib% (
	cd %_nanoguidir%
	msbuild nanogui-obj.vcxproj /property:Configuration=%_compilationTarget%
	copy nanogui-obj.dir\%_compilationTarget%\%_nanoguilib% %_libdir%
)

REM FLATBUFFERS
if not exist %_exedir%\%_flatcexe% (
	cd %_flatbuffersdir%
	msbuild flatc.vcxproj /property:Configuration=%_compilationTarget%
	copy %_compilationTarget%\%_flatcexe% %_exedir%
)

cd %WORKSPACE%
mkdir ..\cpp\include\api
%_exedir%\%_flatcexe% -o ..\cpp\include\api --cpp ..\share\flatbuffers\api.fbs

cd %WORKSPACE%/..

REM
REM
REM	Seascope compilation
REM
REM

REM SEASCOPE
if exist %_libdir%\%_fmtlib% (
	if exist %_libdir%\%_glfwlib% (
		if exist %_libdir%\%_zlib% (
			if exist %_libdir%\%_pnglib% (
					if exist %_libdir%\%_sqlitelib% (
						if exist %_libdir%\%_nanoguilib% (
							msbuild developer/win32/seascope.sln /p:Configuration=%_compilationTarget%;Platform=x64
							REM Copy executable and DLLs
							copy developer\win32\x64\%_compilationTarget%\seascope.exe %_seascopebindir%
							copy %_bindir%\%_pngdll% %_seascopebindir%
							copy %PLUGINS_WORKSPACE%\netCDF\bin\netcdf.dll %_seascopebindir%
							copy %PLUGINS_WORKSPACE%\netCDF\bin\hdf5.dll %_seascopebindir%
							copy %PLUGINS_WORKSPACE%\netCDF\bin\hdf5_hl.dll %_seascopebindir%
							copy %PLUGINS_WORKSPACE%\netCDF\bin\msvcp140.dll %_seascopebindir%
							copy %PLUGINS_WORKSPACE%\netCDF\bin\vcruntime140.dll %_seascopebindir%
							copy %PLUGINS_WORKSPACE%\netCDF\bin\zlib1.dll %_seascopebindir%
						)
					)
			)
		)
	)
)
