// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef COLLECTION_INI_PARSER_H
#define COLLECTION_INI_PARSER_H

#include <map>
#include <vector>
#include <string>

#include "vendors/_ini_parser.h"

#include "core/types.h"
#include "core/structs.h"
#include "core/rendering_cfg.h"

namespace SEAScope
{
	class CollectionINIParser
	{
		public:
			CollectionINIParser();
			virtual ~CollectionINIParser();

			bool load(uint32_t, const std::string&, CollectionCfg&, std::vector<RenderingCfg*>&) const;
            bool validate(const std::string&, INI::Level&) const;

			static std::string trim(const std::string&);
			static std::vector<std::string>& splitString(const std::string&, char, std::vector<std::string>&);
			static bool extractTags(const std::string&, std::map<std::string, std::string>&);
	};
};

#endif
