// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef COLLECTIONS_EXPLORER_H
#define COLLECTIONS_EXPLORER_H

#include <map>
#include <set>
#include <string>
#include <cstdint>

#include "core/types.h"
#include "core/structs.h"
#include "core/rendering_cfg.h"
#include "local/collection_ini_parser.h"
#include "local/granules_index.h"
#include "local/collections_catalogue.h"
#include "local/idf_reader.h"
#include "platform.h"

namespace SEAScope
{
	class CollectionsExplorer
	{
		public:
			CollectionsExplorer();
			virtual ~CollectionsExplorer();

			bool init(const std::string&, const std::string&, bool);
            bool listCollections(const std::string&, std::vector<std::string>&) const;
			bool loadIndex(const std::vector<std::string>&);
			bool registerData(const std::vector<std::string>&, bool);
			bool findGranules(const std::string&, std::uint32_t);
			bool registerCollection(const std::string&, std::uint32_t&);
			bool indexGranule(const std::string&, std::uint32_t);
            bool isIndexUpdateNeeded(const std::string&, const std::string&, const std::vector<std::string>&) const;

			std::string dataPath;
			CollectionINIParser parser;
			GranulesIndex index;
			CollectionsCatalogue catalogue;
			IDFReader reader;
			std::string indexPath;
			bool detectOutdatedIndex;
	};
};

#endif
