// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#ifndef EMBEDDED_CATALOGUE_H
#define EMBEDDED_CATALOGUE_H

#include "core/interfaces/icatalogue_service_connector.h"
#include "local/collections_explorer.h"

#include "platform.h"

namespace SEAScope
{
	class EmbeddedCatalogue final: public virtual ICatalogueServiceConnector
	{
		public:
			EmbeddedCatalogue();
			~EmbeddedCatalogue() override;

			bool init(const Settings&) override;
            bool reset() override;
			bool listCollections(std::set<std::uint32_t>&) const  override;
			bool getCollectionCfg(std::uint32_t, CollectionCfg&, std::vector<RenderingCfg*>&) const  override;
			bool canHandle(const std::string&) const  override;
			static EmbeddedCatalogue* create();
			std::string getPath(const CollectionId& id) const override;

			CollectionsExplorer explorer;
	};
};

#endif
