// This file is part of SEAScope, a 3D visualisation and analysis application
// for satellite, in-situ and numerical model data.
//
// Copyright (C) 2014-2022 OceanDataLab
//
// SEAScope is free software: you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
#include <iostream>

#include "local/embedded_datastream.h"
#include "filter_logger.h"
#include "core/log.h"
#include "core/path.h"
#include "platform/path_impl.h"

using namespace SEAScope;

/**
 *
 */
EmbeddedDatastream::EmbeddedDatastream()
{
};

/**
 *
 */
EmbeddedDatastream::~EmbeddedDatastream()
{
};

/**
 *
 */
bool EmbeddedDatastream::init(const Settings& s)
{
    bool ok(false);

	if
		(nullptr == Log::implementation)
	{
        FilterLogger * logger = new FilterLogger();
        ok = logger->init(s.logFilter);
        if
            (false == ok)
        {
            delete logger;
            logger = nullptr;

            std::cerr << "Failed to initialize logger with '" << s.logFilter << "'" << std::endl;

            return false;
        }
		Log::implementation = logger;
        logger = nullptr;
	}

    if
        (nullptr == Path::implementation)
    {
		Path::implementation = new SEAScope::PathImpl;
    }

#if defined __linux__ || defined __APPLE__
	setenv("TZ", "UTC", 1);
#elif _WIN32
	_putenv_s("TZ", "UTC");
#endif

    return true;
};

/**
 *
 */
bool EmbeddedDatastream::reset()
{
    if
        (nullptr != Log::implementation)
    {
        delete Log::implementation;
		Log::implementation = nullptr;
    }

    if
        (nullptr != Path::implementation)
    {
        delete Path::implementation;
		Path::implementation = nullptr;
    }

    return true;
}

/**
 *
 */
EmbeddedDatastream* EmbeddedDatastream::create()
{
	return new EmbeddedDatastream();
};

/**
 *
 */
bool EmbeddedDatastream::loadData( const GranuleId& /*granuleId*/, const std::string& uri
								 , const std::set<std::string>& fields
								 , DataModel dataModel, GranuleData& result) const
{
	bool ok;	//

    Log::log(LogLevel::Debug, "Embedded datastream", "Load data from {}", uri);
	//ok = IDFReader::extractGranuleData(uri, fields, dataModel, result);
	ok = this->reader.extractGranuleData(uri, fields, dataModel, result);
	return ok;
};

/**
 *
 */
bool EmbeddedDatastream::canHandle(const std::string& uri) const
{
	return "embedded" == uri;
};
