:: =============================================================================
:: @author: <guillaume.leseach@oceandatalab.com>
:: @date: 2019-06-28
:: 
:: This file is part of SEAScope, a 3D visualisation and analysis application
:: for satellite, in-situ and numerical model data.
::
:: Copyright (C) 2014-2022 OceanDataLab
::
:: SEAScope is free software: you can redistribute it and/or modify it under
:: the terms of the GNU Lesser General Public License as published by the Free
:: Software Foundation, either version 3 of the License, or (at your option)
:: any later version.
::
:: SEAScope is distributed in the hope that it will be useful, but WITHOUT ANY
:: WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
:: FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
:: details.
::
:: You should have received a copy of the GNU Lesser General Public License
:: along with SEAScope. If not, see <https://www.gnu.org/licenses/>.
:: =============================================================================
@echo off
setlocal
SET _devdir=%~dp0
SET _devdir=%_devdir:~0,-1%
for %%i in ("%_devdir%") do set _projectdir=%%~dpi
SET _pluginsdir=%_projectdir:~0,-1%

:: Check if arg1 and arg2 exist
if "%1" == "" (
	call :missing_argument 
	goto :eof
) else (
	if '%2%' == '' (
		call :missing_argument 
		goto :eof
	)
	else (
		if '%3%' == '' (
			call :missing_argument 
			goto :eof
		)
	)
)

SET _compilationTarget=release

:: Check if arg1 contains debug or release string
if "%1" == "debug" (
	SET _compilationTarget=Debug
) else (
	if "%1" == "release" (
		SET _compilationTarget=Release
	) else (
		echo Error arg 1. Invalid compilation mode. Use debug or release.
		goto :eof
	)
)

:: Check if path in arg2 is valid
if not exist %2 (
	echo Error arg 2. Invalid path.
	goto :eof
)

:: Set all paths
SET netcdfdir=%2
SET WORKSPACE=%_pluginsdir%\thirdparty
SET SEASCOPE_WORKSPACE=%3
SET SEASCOPE_SRC_DIR=%SEASCOPE_WORKSPACE%\cpp\src
SET SEASCOPE_INC_DIR=%SEASCOPE_WORKSPACE%\cpp\include
SET _bindir=%WORKSPACE%\bin\%_compilationTarget%
SET _exedir=%WORKSPACE%\bin\exe
SET _libdir=%WORKSPACE%\lib\%_compilationTarget%
SET _pluginsbindir=%_pluginsdir%\bin\%_compilationTarget%

:: Create directories
cd %_pluginsdir%
mkdir bin
mkdir %_pluginsbindir%
mkdir thirdparty
mkdir tmp
mkdir tmp\src
mkdir build

cd %WORKSPACE%
mkdir bin
mkdir %_bindir%
mkdir %_exedir%
mkdir lib
mkdir %_libdir%
mkdir tmp
mkdir tmp\src
:: Make a junction with the NetCDF directory in thirdparty directory
mklink /j netCDF %netcdfdir%

:: Set path for client sources
SET CLIENT_SRC_DIR=%_pluginsdir%\tmp\src

:: Set paths for libraries
SET _cmderdir=%WORKSPACE%\cmder
SET _fmtdir=%WORKSPACE%\fmt
SET _glmdir=%WORKSPACE%\glm
SET _netcdfdir=%WORKSPACE%\netCDF
SET _netcdfcxxdir=%WORKSPACE%\netcdf-cxx4
SET _flatbuffersdir=%WORKSPACE%\flatbuffers
SET _tmpdir=%WORKSPACE%\tmp

::
:: Libraries name
::

:: Set right name for fmt lib
if %_compilationTarget% == Release (
	SET _fmtlib=fmt.lib
) else (
	if %_compilationTarget% == Debug (
		SET _fmtlib=fmtd.lib
	)
)

SET _netcdfcxxlib=netcdf-cxx4.lib
SET _flatcexe=flatc.exe

:: git url for libraries
SET _fmturl=https://github.com/fmtlib/fmt.git
SET _glmurl=https://github.com/g-truc/glm.git
SET _iniparserurl=https://github.com/Poordeveloper/ini-parser.git
SET _netcdfcxx4url=https://github.com/Unidata/netcdf-cxx4.git
SET _flatbuffersurl=https://github.com/google/flatbuffers.git

::
::
::	Downloads
::
::

:: FMT
cd %WORKSPACE%
if not exist %_fmtdir% (
	git clone %_fmturl% %_fmtdir%
	cd %_fmtdir%
	git checkout d561cb47a76ea0563721809dfbd4f717741f21ca
	cmake -G "Visual Studio 17 2022" -A x64 .
)

:: INI-PARSER
cd %WORKSPACE%
if not exist ini-parser (
	git clone %_iniparserurl% ini-parser
	cd ini-parser
	git checkout 6f65f735d116a735a768f36232e978dee4a33e42
)

:: FLATBUFFERS
cd %WORKSPACE%
if not exist %_flatbuffersdir% (
	git clone %_flatbuffersurl% %_flatbuffersdir%
	cd %_flatbuffersdir%
	git checkout b2ce86ef8aef9a9caa8bcc99e54591b92d7b2426
	cmake -G "Visual Studio 17 2022" -A x64 .
	powershell -Command "(Get-Content flatc.vcxproj) | Foreach-Object {$_ -replace '<TreatWarningAsError>true</TreatWarningAsError>','<TreatWarningAsError>false</TreatWarningAsError>'} | Out-File flatc.vcxproj"
)

:: GLM
cd %WORKSPACE%
if not exist %_glmdir% (
	git clone "%_glmurl%" glm
	cd glm
	git checkout 0.9.9.0
)

:: NETCDF-CXX
cd %WORKSPACE%
if not exist %_netcdfcxxdir% (
	git clone %_netcdfcxx4url% %_netcdfcxxdir%
	cd %_netcdfcxxdir%
	:: Patch CMakeLists.txt to remove useless HDF5 check
	patch -u CMakeLists.txt -i %_pluginsdir%\developer\win32\netcdf-cxx4_CMakeLists.patch
	cmake -G "Visual Studio 17 2022" -A x64 -DBUILD_SHARED_LIBS=OFF .
)

::
::
::	Compilation
::
::

:: FMT
if not exist %_libdir%\%_compilationTarget%\%_fmtlib% (
	cd %_fmtdir%
	msbuild fmt.vcxproj /property:Configuration=%_compilationTarget%
	copy %_compilationTarget%\%_fmtlib% %_libdir%
)

:: NETCDF-CXX
if not exist %_libdir%\%_compilationTarget%\%_netcdfcxxlib% (
	cd %_netcdfcxxdir%
	msbuild cxx4\netcdf-cxx4.vcxproj /property:Configuration=%_compilationTarget%
	copy cxx4\%_compilationTarget%\%_netcdfcxxlib% %_libdir%
)

::
::
::	Src copy
::
::

copy "%SEASCOPE_SRC_DIR%\core\rendering_cfg.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\log.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\simplelogger.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\filter_logger.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\..\win32\src\path_impl.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\path.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\data\image.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\data\events_coverage.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\settings.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\timeline_cfg.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\local\granules_index.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\local\collections_catalogue.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\utils.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\api_in.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\api_out.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\strptime.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\icatalogue_service_connector.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\idatastream_service_connector.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\iindex_service_connector.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\ilog_handler.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\filter_logger.cpp" "%CLIENT_SRC_DIR%"
copy "%SEASCOPE_SRC_DIR%\core\ipath_handler.cpp" "%CLIENT_SRC_DIR%"

:: Script exit
EXIT /B 0

::
:: Function
::

:missing_argument
echo An argument is missing.
echo First argument is compilation mode (debug / release)
echo Second argument is NetCDF path
echo Third argument is SEAScope dir path
EXIT /B 0
